<?php
namespace Koob\BaccaratBundle\Model;

use Koob\InterfaceBundle\BaccaratInterface;
use Koob\InterfaceBundle\DatabaseInterface;
use Koob\GameBundle\Model\GameModel;

class FunctionModel extends GameModel implements BaccaratInterface
{
	protected $database = 'baccarat';

	protected $attrs = ['min', 'max', 'reply', 'depo', 'allDepo', 'setba', 'deli', 'allDeli', 'recc', 'sc', 'cp', '$'];

	public function rebuildBaccarat(DatabaseInterface $con)
	{
		$con->startQuery()
			->drop()
			->table($this->database)
			->build()
			->query()
			->close();

		$con->startQuery()
			->create([
				'name'     => 'VARCHAR(255)',
				'userId'   => 'VARCHAR(255)  UNIQUE',
				'admin'    => 'VARCHAR(255)',
				'content'  => 'TEXT',
				'result'   => 'TEXT',
				'rollBack' => 'TEXT',
			])
			->table($this->database)
			->build()
			->query()
			->close();

		$datas = [
			'status' => 'down',
			'game'   => 0,
			'bigLap' => 1,
			'lap'    => 1,
			'min'    => 20,
			'max'    => 200,
			'reply'  => 'yes',
		];
		$data = json_encode($datas);
		$con->startQuery()
			->insert([
				'name' => '"index"',
				'userId' => 'index',
				'content' => $data,
			])
			->into($this->database)
			->build()
			->query()
			->close();
	}

	public function wage(DatabaseInterface $con, $text)
	{
		$hasDeli = stripos($text, $this->datas['deli']);
		if ($this->datas['status'] == 'down') {
			return;
		} else if ($this->datas['status'] == 'close') {
			return 'ยังไม่เปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			return 'แทงไม่ทัน';
		} else if (!$hasDeli) {
			return "รูปแบบการแทง xx".$this->datas['deli']."xxx";
		}
		$explodeTexts = explode($this->datas['allDeli'], $text);
		foreach ((array) $explodeTexts as $wageList)
		{
			[$type, $wageNet] = explode($this->datas['deli'], $wageList);
			if (strlen($type) != 1) {
				$errors[] = "การแทง $wageList การแทงผิด";
				continue;
			}
			if (strlen($type) == 1 && !in_array($type, [1, 2, 3, 4, 5])) {
				$errors[] = "การแทง $wageList การแทงผิด";
				continue;
			}
			if (!is_numeric($wageNet)) {
				$errors[] = "การแทง $wageList ยอดเงินผิด";
				continue;
			}
			if ($this->datas['min'] > $wageNet || $wageNet > $this->datas['max']) {
				$errors[] = "การแทง $wageList ผิด จำนวนเงินตั้งแต่ ".$this->datas['min']."-".$this->datas['max'];
				continue;
			}
			if (!empty($this->datas['depo']) && !empty($this->bonus) && $this->datas['depo'] > 0) {
				$max = $this->deposite*$this->datas['depo']/100;
				$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
				if ($wageNet < $maximum) {
					$errors[] = "แทง ".$this->datas['depo']."% ของยอดฝาก($max ขึ้นไป)";
					continue;
				}
			}
			if (!empty($this->datas['cp']) && $this->userResult != 'playing' && $this->checkPeople($con, $this->database, true) >= $this->datas['cp']) {
				$errors[] = "จำนวนคนครบ ".$this->datas['cp']." คนแล้ว";
				continue;
			}
			$this->contents[$type] = $wageNet;
		}
		foreach ((array) $this->contents as $wage)
		{
			$allWageNet += $wage;
		}
		if (!empty($this->datas['allDepo']) && !empty($this->bonus) && $this->datas['allDepo'] > 0)  {
			$max = $this->deposite*$this->datas['allDepo']/100;
			$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
			if ($allWageNet < $maximum) {
				$errors[] = "แทงทั้งหมด ".$this->datas['allDepo']."% ของยอดฝาก($max ขึ้นไป)";
			}
		}
		if ($allWageNet <= $this->net+$this->bonus) {
			$errorText = implode(" ", (array) $errors);
		} else {
			$errorText = "เครดิตไม่เพียงพอ❗";
		}
		$clearErrorText = str_replace([' ', "\r\n"], '', $errorText);
		if (empty($clearErrorText)) {
			$content = $this->codename($this->contents);
			$con->startQuery()
				->update(['content' => $content, 'result' => 'playing'])
				->set($this->database)
				->where(['userId' => $this->userId])
				->build()
				->query()
				->close();
		}
		$replyTexts = $this->check();
		if ($this->datas['reply'] == 'yes') {
			if (!empty($errorText)) {
				$replyTexts .= "\r\n\r\n$errorText";
			}
		} else if ($this->datas['reply'] == 'flex') {
			if (!empty($clearErrorText)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							$errorText,
							'separator #000000',
						],
					],
				], 'xs');
			}
		} else {
			unset($replyTexts);
		}
		return $replyTexts;
	}

	public function unWage(DatabaseInterface $con, $text)
	{
		if ($this->datas['status'] != 'open') {
			return $this->datas['status'] == 'down' ? '' : "$this->name ปิดรอบแล้ว❗";
		}
		foreach ((array) $this->contents as $type => $wage)
		{
			unset($this->contents[$type]);
			$cancel .= " $type";
		}
		$content = json_encode($this->contents);
		$con->startQuery()
			->update(['content' => $content, 'result' => null])
			->set($this->database)
			->where(['userId' => $this->userId])
			->build()
			->query()
			->close();
		if (!empty($cancel)) {
			if ($this->datas['reply'] == 'flex') {
				$replyTexts = $this->flexMessage([
					'old' => $this->check(),
					'body' => [
						[
							"ยกเลิกขา$cancel",
							"separator #000000",
						],
					],
				], 'xs');
			} else if ($this->datas['reply'] == 'yes') {
				$replyTexts = $this->check()."\r\n\r\nยกเลิกขา$cancel";
			}
		}
		return $replyTexts;
	}

	public function unWageAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		$con->startQuery()
			->update(['content' => '{}', 'result' => null])
			->set($this->database)
			->where('id>1')
			->build()
			->query()
			->close();
		return "ยกเลิกการแทงทั้งหมด";
	}

	public function check()
	{
		$replyTexts = $this->flexMessage([
			'altText' => "สรุปการแทงของ $this->name",
			'header' => "สรุปการแทงของ $this->name",
			'footer' =>	"{$this->net}💸{$this->bonus}",
			'body' => [
				[
					$this->name,
				],
			],
		], 'xs');
		$isEmpty = 1;
		$text = "$this->name แทง";
		foreach ((array) $this->contents as $type => $wageNet)
		{
			$isEmpty = 0;
			if ($type == 1) $type = "p";
			if ($type == 2) $type = "b";
			if ($type == 3) $type = "เสมอ";
			if ($type == 4) $type = "คู่เหมือนน้ำเงิน";
			if ($type == 5) $type = "คู่เหมือนแดง";
			$text .= " $type=$wageNet";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						$type,
						$wageNet,
					],
				],
			], 'xs');
		}
		if ($isEmpty) {
			$text .= "ไม่ได้แทง";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					'ไม่ได้แทง',
				],
			], 'xs');
		}
		$text .= "\r\n{$this->net}💸{$this->bonus}";
		$replyTexts = $this->flexMessage([
			'old' => $replyTexts,
			'body' => [
				[
					'separator #000000',
				],
			],
		]);
		if ($this->datas['reply'] == 'yes') {
			$replyTexts = $text;
		}
		return $replyTexts;
	}

	public function checkAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}

		$resultRows = $con->startQuery()
			->select(['x.name, x.content, m.userId', 'm.net', 'm.bonus'])
			->from($this->database.' x, money.money m')
			->where('x.userId = m.userId')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$replyTexts = $this->flexMessage([
			'altText' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
			'header' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
		]);
		foreach ((array) $resultRows as $rows)
		{
			$contents = json_decode($rows['content'], 1);
			$name = json_decode($rows['name'], 1);
			foreach ((array) $contents as $type => $wageNet)
			{
				$reply .= " $type=$wageNet";
			}
			if ($rows['bonus'] > 0) {
				$bonusText = "💸".$rows['bonus']."=".($rows['bonus']+$rows['net']);
			}
			if (!empty($reply)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							"$name แทง$reply 💰".$rows['net']."{$bonusText}{$suffix}",
						],
					],
				]);
			}
			unset($reply, $bonusText);
		}
		$noPlayerReplys = [
			"type" => "text",
			"text" => "ยังไม่มีผู้เล่นแทง",
		];
		if (empty($replyTexts['messages'][0]['contents']['contents'][0]['body'])) {
			$replyTexts = [
				'messages' => [$noPlayerReplys],
				'raw' => true
			];
		}
		return $replyTexts;
	}

	public function openGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'down') {
			$sendText = $this->resultOpen($con, $this->database, $this->datas['game'], $resetTime);
			$game = $this->datas['game'] + 1;
			$replyText = "เริ่มเกมที่ #".$game;
			$this->datas['status'] = 'close';
			$this->datas['game'] = $game;
			$this->datas['bigLap'] = 1;
			$this->datas['lap'] = 1;
			$this->datas['startTime'] = date('H.i');
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else {
			$replyText = "เปิดบ้านแล้ว";
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function openRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			// $replyText[] = $locals['openMessage'];
			// $replyText[] = "เปิดรอบย่อยที่ #".$this->datas['lap'];
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			if ($this->datas['sc'] != 'no' && !empty($this->scores)) {
				$replyText["messages"][] = $this->showScore($con, $this->database)['messages'][0];
			}
			$this->datas['status'] = 'open';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'เปิดรอบแล้ว';
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'กรุณาสรุปผลให้เรียบร้อยก่อน';
		}
		return $replyText;
	}

	public function closeRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'open') {
			// $replyText[] = $locals['closeMessage'];
			// $replyText[] = 'ปิดรอบย่อยที่ #' . $this->datas['lap'];
			$checkAll = $this->checkAll($con);
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				$checkAll["messages"][0],
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			$this->datas['status'] = 'check';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'rollBack' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'ปิดรอบแล้ว';
		}
		return $replyText;
	}

	public function restRound(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultRest($con, $this->database, $this->datas['game'], $this->datas['bigLap'], $resetTime, $this->datas['startTime']);
			$bigLap = $this->datas['bigLap'];
			$replyText = 'ปิดรอบใหญ่ที่ #'.$bigLap;
			$this->datas['lap'] = 1;
			$this->datas['bigLap'] = $bigLap + 1;
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'admin' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['count' => 0, 'wage' => 0, 'deposite' => 0])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else {
			$replyText = 'โปรดเช็คสถานะ';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function closeGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultClose($con, $this->database, $this->datas['game'], $resetTime);
			$replyText = 'ปิดบ้านแล้ว';
			$this->rebuildReport($con, $this->database);
			$this->datas['status'] = 'down';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'admin' => null, 'rollBack' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['tag' => 0, 'count' => 0, 'wage' => 0, 'deposite' => 0, 'rollBack' => null])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'ยังไม่ปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			$replyText = 'ยังไม่สรุป';
		} else {
			$replyText = 'ปิดบ้านแล้ว';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function saveResult(DatabaseInterface $con, $text)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] == 'down' || $this->datas['status'] == 'close') {
			return;
		}
		if ($this->datas['status'] != 'check') {
			return 'กรุณาปิดรอบก่อน';
		}
		$text = substr($text, 1);
		list($player, $banker) = explode(',', $text);
		if ((strlen($player) > 3 && stripos($player, '10') === false) || (strlen($banker) > 3 && stripos($banker, '10') === false)) {
			$error = 1;
		}
		if ((stripos($player, '10') == 0 && strlen($player) > 6) || (stripos($banker, '10') == 0 && strlen($banker) > 6)) {
			$error = 1;
		}
		for ($i=0; $i<strlen($player) || $i<strlen($banker); $i++)
		{
			if (!empty($player[$i]) && !is_numeric($player[$i]) && !in_array($player[$i], ['j', 'q', 'k'])) {
				$error = 1;
			} else {
				if ($player[$i] != '0') $playerValue += (int) $player[$i];
				else $playerValue -= 1;
			}
			if (!empty($banker[$i]) && !is_numeric($banker[$i]) && !in_array($banker[$i], ['j', 'q', 'k'])) {
				$error = 1;
			} else {
				if ($banker[$i] != '0') $bankerValue += (int) $banker[$i];
				else $bankerValue -= 1;
			}
		}
		if (($player[0] == $player[1] || substr($player, 0, 4) == '1010') && $player[2] != '0') {
			$isPlayerSame = 1;
		}
		if (($banker[0] == $banker[1] || substr($banker, 0, 4) == '1010') && $banker[2] != '0') {
			$isBankerSame = 1;
		}
		$playerValue %= 10;
		$bankerValue %= 10;
		$results = [
			'player' => [
				'isSame' => (int) $isPlayerSame,
				'value' => $playerValue,
			],
			'banker' => [
				'isSame' => (int) $isBankerSame,
				'value' => $bankerValue,
			],
		];
		$replyText = "p$playerValue แต้ม, b$bankerValue แต้ม";
		if ($error == 1) {
			$replyText = "สรุปผิด";
		} else {
			$replyText = "สรุป $replyText\r\n\r\nยันยืนผลสรุป yes หรือ @ok";
			$result = json_encode($results);
			$con->startQuery()
				->update(['result' => $result])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function submit(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] != 'check') {
			return;
		}
		if (empty($this->results)) {
			return 'สรุปผิด';
		} else {
			$con->startQuery()
				->update(['rollBack' => null])
				->set($this->database)
				->build()
				->query()
				->close();
			$recieveDatabases = $con->startQuery()
				->select(['id', 'content'])
				->from($this->database)
				->where('content IS NOT NULL')
				->and('content != ""')
				->and('content != "{}"')
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			foreach ((array) $recieveDatabases as $rows)
			{
				$con->startQuery()
					->update(['rollBack' => $rows['content']])
					->set($this->database)
					->where(['id' => $rows['id']])
					->build()
					->query()
					->close();
			}
			$con->startQuery()
				->update(['rollBack' => null])
				->set('money', 'money')
				->build()
				->query()
				->close();
			$replyText = $this->result($con, $this->database);
			$this->datas['lap']++;
			$this->datas['status'] = 'close';
			unset($this->results);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'result' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function setup(DatabaseInterface $con, $admins, $force)
	{
		if (!in_array($this->userId, $admins)) {
			return;
		}
		$database = $con->getDatabase();

		$existedDatabases = $con->startQuery()
			->select('SCHEMA_NAME')
			->from('SCHEMATA', 'information_schema')
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array($database, $existedDatabases)) {
			$con->startQuery()
				->create()
				->database($database)
				->build()
				->query()
				->close();
		}

		if (!in_array('money', $existedDatabases)) {
			$con->startQuery()
				->create()
				->database('money')
				->build()
				->query()
				->close();
		}

		$allTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => $database])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		$moneyTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => 'money'])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array('money', $moneyTables)) {
			$this->rebuildMoney($con);
			$install .= ' money';
		}
		if (!in_array('reportMoney', $moneyTables)) {
			$this->rebuildReportMoney($con);
			$install .= ' reportMoney';
		}
		if (!in_array('login', $allTables)) {
			$this->rebuildLogin($con);
			$install .= ' login';
		}
		if ($force === true || !in_array($this->database, $allTables)) {
			$this->rebuildBaccarat($con);
			$install .= ' baccarat';
		}
		if ($force === true || !in_array('reportBaccarat', $allTables)) {
			$this->rebuildReport($con, $this->database);
			$install .= ' reportBaccarat';
		}
		if ($force === true || !in_array('reportAllBaccarat', $allTables)) {
			$this->rebuildAllReport($con, $this->database);
			$install .= ' reportAllBaccarat';
		}

		if (!empty($install)) {
			$replyText = "ติดตั้ง$install สำเร็จ";
		} else {
			$replyText = "มีฐานข้อมูลอยู่แล้ว";
		}
		return $replyText;
	}

	protected function saveScore(DatabaseInterface $con)
	{
		$player = $this->results['player']['value'];
		$banker = $this->results['banker']['value'];
		if ($player > $banker) {
			$nextScore = 1;
		} else if ($player < $banker) {
			$nextScore = 2;
		} else {
			$nextScore = 3;
		}
		$this->scores[] = $nextScore;
		$score = json_encode($this->scores);
		$con->startQuery()
			->update(['admin' => $score])
			->set($this->database)
			->where(['id' => 1])
			->build()
			->query()
			->close();
	}

	protected function rule()
	{
		$setba = is_null($this->datas['setba']) ? 0.9 : 1-($this->datas['setba'])/100;
		$rewards = [
			1 => 1,
			2 => $setba,
			3 => 8,
			4 => 11,
			5 => 11,
		];
		if ($this->results['player']['value'] > $this->results['banker']['value']) {
			$results[] = 1;
		} else if ($this->results['player']['value'] < $this->results['banker']['value']) {
			$results[] = 2;
		} else {
			$results[] = 3;
			$reduces[1] = 0;
			$reduces[2] = 0;
		}
		if ($this->results['player']['isSame'] == 1) {
			$results[] = 4;
		}
		if ($this->results['banker']['isSame'] == 1) {
			$results[] = 5;
		}
		return [$rewards, $results, $reduces];
	}
}