<?php
namespace Koob\DatabaseBundle\Storage;

use Koob\InterfaceBundle\DatabaseInterface;

class DatabaseStorage implements DatabaseInterface
{

	protected $host;

	protected $user;

	protected $pass;

	protected $database;

	protected $con;

	protected $table;

	protected $header;

	protected $body;

	protected $condition;

	protected $sql;

	protected $result;

	protected $error = null;

	protected $errorLocation = __DIR__.'/../../../logs/database_error_logs.log';

	public function setHost($host)
	{
		$this->host = $host;
	}

	public function getHost()
	{
		return $this->host;
	}

	public function setUser($user)
	{
		$this->user = $user;
	}

	public function getUser()
	{
		return $this->user;
	}

	public function setPass($pass)
	{
		$this->pass = $pass;
	}

	public function getPass()
	{
		return $this->pass;
	}

	public function setDatabase($database)
	{
		$this->database = $database;
	}

	public function getDatabase()
	{
		return $this->database;
	}

	public function startQuery()
	{
		$this->con = new \mysqli($this->host, $this->user, $this->pass);
		if ($this->con->connect_error) {
			$this->errorLog('Construct error: '.$this->con->connect_error);
		}
	}

	public function create($creates = null)
	{
		$this->body = empty($creates['create']) ? " CHARACTER SET utf8 COLLATE utf8_general_ci" : "(".$creates['create'].")";
	}

	public function select($selects)
	{
		$this->body = $selects['select'];
	}

	public function update($updates)
	{
		$this->body = $updates['update'];
	}

	public function insert($inserts)
	{
		$this->body = "(".$inserts['insertField'].") VALUES (".$inserts['insertValue'].")";
	}

	public function drop()
	{
		$this->body = ' ';
	}

	public function delete()
	{
		$this->body = ' ';
	}

	public function from($table, $database = null)
	{
		$database = empty($database) ? $this->database : $database;
		$this->table = " FROM $database.$table ";
	}

	public function set($table, $database = null)
	{
		$database = empty($database) ? $this->database : $database;
		$this->table = "$database.$table SET ";
	}

	public function into($table, $database = null)
	{
		$database = empty($database) ? $this->database : $database;
		$this->table = "INTO $database.$table ";
	}

	public function database($database)
	{
		$this->table = "DATABASE $database";
	}

	public function table($table, $database = null)
	{
		$database = empty($database) ? $this->database : $database;
		$this->table = "TABLE $database.$table";
	}

	public function where($where)
	{
		$this->condition = ' WHERE '.$where;
	}

	public function and($and)
	{
		$this->condition .= ' AND '.$and;
	}

	public function or($or)
	{
		$this->condition .= ' OR '.$or;
	}

	public function like($like)
	{
		$this->condition .= ' LIKE '.$like;
	}

	public function method($method)
	{
		$this->method = $method;
		switch ($method)
		{
			case 'create':
				$this->header = 'CREATE ';
				break;

			case 'select':
				$this->header = 'SELECT ';
				break;

			case 'update':
				$this->header = 'UPDATE ';
				break;

			case 'insert':
				$this->header = 'INSERT ';
				break;

			case 'drop':
				$this->header = 'DROP ';
				break;

			case 'delete':
				$this->header = 'DELETE ';
				break;

			default:
				$this->errorLog("Method $method not found.");
				break;
		}
	}

	public function build()
	{
		if (empty($this->header)) {
			$this->errorLog('Missing header');
		}
		if (empty($this->table)) {
			$this->errorLog('Missing table');
		}
		if (empty($this->body)) {
			$this->errorLog('Missing body');
		}
		switch ($this->method)
		{
			case 'create':
				$this->sql = $this->header.$this->table.$this->body;
				break;

			case 'select':
				$this->sql = $this->header.$this->body.$this->table.$this->condition;
				break;

			case 'update':
				$this->sql = $this->header.$this->table.$this->body.$this->condition;
				break;

			case 'insert':
				$this->sql = $this->header.$this->table.$this->body;
				break;

			case 'drop':
				$this->sql = $this->header.$this->table;
				break;

			case 'delete':
				$this->sql = $this->header.$this->table.$this->condition;
				break;

			default:
				break;
		}
	}

	public function query(bool $isReturnBool = false)
	{
		$this->con->query('SET NAMES utf8');
		try {
			if (empty($this->con)) {
				throw new \Exception('Empty connection.');
			} else if (empty($this->sql)) {
				throw new \Exception('Empty sql.');
			} else if (!empty($this->error)) {
				throw new \Exception('Something error. SQL for query => '.$this->sql);
			} else if (!$this->result = $this->con->query($this->sql)) {
				throw new \Exception('query error. => '.$this->con->error.' (query) '.$this->sql);
			}
		} catch (\Exception $e) {
			$this->errorLog($e->getMessage());
			return false;
		}
		return true;
	}

	public function fetchAssoc(bool $isOneArray = false)
	{
		if (!$this->result) {
			return;
		}
		while ($result = $this->result->fetch_assoc())
		{
			$return[] = $isOneArray ? $result[array_keys($result)[0]] : $result;
		}
		return (array) $return;
	}

	public function getSql()
	{
		return $this->sql;
	}

	public function close()
	{
		unset($this->header, $this->body, $this->table, $this->condition, $this->sql, $this->result, $this->error);
		$this->con->close();
	}

	protected function errorLog($error)
	{
		$error = empty($error) ? 'No error message.' : $error;
		$this->error .= empty($this->error) ? $error : ", $error";
		error_log('{'.date("Y-m-d H:i:s").'} '.$error."\n", 3, $this->errorLocation);
	}
}