<?php
namespace Koob\HiloBundle\Model;

use Koob\InterfaceBundle\HiloInterface;
use Koob\InterfaceBundle\DatabaseInterface;
use Koob\GameBundle\Model\GameModel;

class FunctionModel extends GameModel implements HiloInterface
{
	protected $database = 'hilo';

	protected $attrs = ['min', 'max', 'reply', 'depo', 'allDepo', 'rule', 'deli', 'allDeli', 'recc', 'cp', '$'];

	public function rebuildHilo(DatabaseInterface $con)
	{
		$con->startQuery()
			->drop()
			->table($this->database)
			->build()
			->query()
			->close();

		$con->startQuery()
			->create([
				'name'     => 'VARCHAR(255)',
				'userId'   => 'VARCHAR(255) UNIQUE',
				'admin'    => 'VARCHAR(255)',
				'content'  => 'TEXT',
				'result'   => 'TEXT',
				'rollBack' => 'TEXT',
			])
			->table($this->database)
			->build()
			->query()
			->close();

		$datas = [
			'status' => 'down',
			'game'   => 0,
			'bigLap' => 1,
			'lap'    => 1,
			'min'    => 20,
			'max'    => 200,
			'reply'  => 'yes',
		];
		$data = json_encode($datas);
		$con->startQuery()
			->insert([
				'name' => '"index"',
				'userId' => 'index',
				'content' => $data,
			])
			->into($this->database)
			->build()
			->query()
			->close();
	}

	public function wage(DatabaseInterface $con, $text)
	{
		$hasDeli = stripos($text, $this->datas['deli']);
		if ($this->datas['status'] == 'down') {
			return;
		} else if ($this->datas['status'] == 'close') {
			return 'ยังไม่เปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			return 'แทงไม่ทัน';
		} else if (!$hasDeli) {
			return "รูปแบบการแทง xx".$this->datas['deli']."xxx";
		}
		$explodeTexts = explode($this->datas['allDeli'], $text);
		foreach ((array) $explodeTexts as $wageList)
		{
			[$type, $wageNet] = explode($this->datas['deli'], $wageList);
			$firstCharecter = mb_substr($type, 0, 1);
			$secondCharecter = mb_substr($type, 1);
			$size = strlen($type);
			if ((int) $this->datas['rule'] == 0) {
				if (is_numeric($type)) {
					if (1 <= $size && $size <= 3) {
						$wages = str_split($type);
						sort($wages);
						$type = implode('', $wages);
						if ($size == 3 && !in_array($type, [111, 222, 333, 444, 555, 666, 123, 456])) {
							$errors[] = "แทง 3 ตัวได้แค่ ตอง 123 และ 456";
							continue;
						}
					} else {
						$errors[] = "$wageList แทงได้ 1-3 ตัว";
						continue;
					}
				} else if (is_numeric($firstCharecter) && !in_array($secondCharecter, ['ต', 'ส'])) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				} else if (in_array($firstCharecter, ['ต', 'ส']) && !empty($secondCharecter) && ($secondCharecter == 0 || $secondCharecter > 6)) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				} else if ($size > 3 && is_numeric($type)) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				} else if (in_array($firstCharecter, ['h']) && !empty($secondCharecter)) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				}
			} else if ((int) $this->datas['rule'] == 1) {
				if (is_numeric($type)) {
					if (1 <= $size && $size <= 3) {
						$wages = str_split($type);
						sort($wages);
						$type = implode('', $wages);
					} else {
						$errors[] = "$wageList แทงได้ 1-3 ตัว";
						continue;
					}
				} else if (is_numeric($firstCharecter) && !in_array($secondCharecter, ['ต', 'ส']) && $type != '11ฮ' && stripos($type, 'สป') == 0) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				} else if (in_array($firstCharecter, ['ต', 'ส']) && !empty($secondCharecter) && ($secondCharecter == 0 || $secondCharecter > 6)) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				} else if ($size > 3 && is_numeric($type)) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				} else if (!in_array(stripos($type, 'สป'), [0, 3])) {
					$errors[] = "แทงขาผิด $wageList";
					continue;
				} else if (stripos($type, 'สป') == 3) {
					if (!in_array(substr($type, 0, 3), [111, 222, 333, 444, 555, 666])) {
						$wages = str_split(substr($type, 0, 3));
						sort($wages);
						$type = implode('', $wages);
						$type .= "สป";
					} else {
						$errors[] = "แทงขาผิด $wageList";
						continue;
					}
				} else if (in_array($firstCharecter, ['h'])) {
					return;
				}
			}
			if (!($this->datas['min'] <= $wageNet && $wageNet <= $this->datas['max'])) {
				$errors[] = "การแทง $wageList ไม่ถูกต้อง วงเงินการแทง ".$this->datas['min'].'-'.$this->datas['max'].' บาท❗';
				continue;
			}
			if (!empty($this->datas['depo']) && !empty($this->bonus) && $this->datas['depo'] > 0) {
				$max = $this->deposite*$this->datas['depo']/100;
				$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
				if ($wageNet < $maximum) {
					$errors[] = "แทง ".$this->datas['depo']."% ของยอดฝาก($max ขึ้นไป)";
					continue;
				}
			}
			if (!empty($this->datas['cp']) && $this->play != 'playing' && $this->checkPeople($con, $this->database, true) >= $this->datas['cp']) {
				$errors[] = "จำนวนคนครบ ".$this->datas['cp']." คนแล้ว";
				continue;
			}
			$this->contents[$type] = $wageNet;
		}
		foreach ((array) $this->contents as $wageNet)
		{
			$allWageNet += $wageNet;
		}
		if (!empty($this->datas['allDepo']) && !empty($this->bonus) && $this->datas['allDepo'] > 0)  {
			$max = $this->deposite*$this->datas['allDepo']/100;
			$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
			if ($allWageNet < $maximum) {
				$errors[] = "แทงทั้งหมด ".$this->datas['allDepo']."% ของยอดฝาก($max ขึ้นไป)";
			}
		}
		if ($allWageNet <= $this->net+$this->bonus) {
			$errorText = implode(" ", (array) $errors);
		} else {
			$errorText = "เครดิตไม่เพียงพอ❗";
		}
		$clearErrorText = str_replace([' ', "\r\n"], '', $errorText);
		if (empty($clearErrorText)) {
			$content = $this->codename($this->contents);
			$con->startQuery()
				->update(['content' => $content, 'result' => 'playing'])
				->set($this->database)
				->where(['userId' => $this->userId])
				->build()
				->query()
				->close();
		}
		$replyTexts = $this->check();
		if ($this->datas['reply'] == 'yes') {
			if (!empty($errorText)) {
				$replyTexts .= "\r\n\r\n$errorText";
			}
		} else if ($this->datas['reply'] == 'flex') {
			if (!empty($clearErrorText)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							$errorText,
							'separator #000000',
						],
					],
				], 'xs');
			}
		} else {
			unset($replyTexts);
		}
		return $replyTexts;
	}

	public function unWage(DatabaseInterface $con, $text)
	{
		if ($this->datas['status'] != 'open') {
			return $this->datas['status'] == 'down' ? '' : "$this->name ปิดรอบแล้ว❗";
		}
		foreach ((array) $this->contents as $type => $wage)
		{
			unset($this->contents[$type]);
			$cancel .= " $type";
		}
		$content = $this->codename($this->contents);
		$con->startQuery()
			->update(['content' => $content, 'result' => null])
			->set($this->database)
			->where(['userId' => $this->userId])
			->build()
			->query()
			->close();
		if (!empty($cancel)) {
			if ($this->datas['reply'] == 'flex') {
				$replyTexts = $this->flexMessage([
					'old' => $this->check(),
					'body' => [
						[
							"ยกเลิกขา$cancel",
							"separator #000000",
						],
					],
				], 'each', 'xs');
			} else if ($this->datas['reply'] == 'yes') {
				$replyTexts = $this->check()."\r\n\r\nยกเลิกขา$cancel";
			}
		}
		return $replyTexts;
	}

	public function unWageAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		$con->startQuery()
			->update(['content' => '{}', 'result' => null])
			->set($this->database)
			->where('id>1')
			->build()
			->query()
			->close();
		return "ยกเลิกการแทงทั้งหมด";
	}

	public function check()
	{
		$replyTexts = $this->flexMessage([
			'altText' => "สรุปการแทงของ $this->name",
			'footer' =>	"{$this->net}💸{$this->bonus}",
			'body' => [
				[
					$this->name,
				],
			],
		], 'xs');
		$isEmpty = 1;
		$text = "$this->name แทง";
		foreach ((array) $this->contents as $type => $wageNet)
		{
			$isEmpty = 0;
			$text .= " $type=$wageNet";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						$type,
						$wageNet,
					],
				],
			], 'xs');
		}
		if ($isEmpty) {
			$text .= "ไม่ได้แทง";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						'ไม่ได้แทง',
					],
				],
			], 'xs');
		}
		$text .= "\r\n{$this->net}💸{$this->bonus}";
		$replyTexts = $this->flexMessage([
			'old' => $replyTexts,
			'body' => [
				[
					'separator #000000',
				],
			],
		]);
		if ($this->datas['reply'] == 'yes') {
			$replyTexts = $text;
		}
		return $replyTexts;
	}

	public function checkAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}

		$resultRows = $con->startQuery()
			->select(['x.name, x.content, m.userId', 'm.net', 'm.bonus'])
			->from($this->database.' x, money.money m')
			->where('x.userId = m.userId')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$replyTexts = $this->flexMessage([
			'altText' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
			'header' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
		]);
		foreach ((array) $resultRows as $rows)
		{
			$contents = json_decode($rows['content'], 1);
			$name = json_decode($rows['name'], 1);
			foreach ((array) $contents as $type => $wageNet)
			{
				$reply .= " $type=$wageNet";
			}
			if ($rows['bonus'] > 0) {
				$bonusText = "💸".$rows['bonus']."=".($rows['bonus']+$rows['net']);
			}
			if (!empty($reply)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							"$name แทง$reply 💰".$rows['net']."{$bonusText}{$suffix}",
						],
					],
				]);
			}
			unset($reply, $bonusText);
		}
		$noPlayerReplys = [
			"type" => "text",
			"text" => "ยังไม่มีผู้เล่นแทง",
		];
		if (empty($replyTexts['messages'][0]['contents']['contents'][0]['body'])) {
			$replyTexts = [
				'messages' => [$noPlayerReplys],
				'raw' => true
			];
		}
		return $replyTexts;
	}

	public function openGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'down') {
			$sendText = $this->resultOpen($con, $this->database, $this->datas['game'], $resetTime);
			$game = $this->datas['game'] + 1;
			$replyText = "เริ่มเกมที่ #".$game;
			$this->datas['status'] = 'close';
			$this->datas['game'] = $game;
			$this->datas['bigLap'] = 1;
			$this->datas['lap'] = 1;
			$this->datas['startTime'] = date('H.i');
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else {
			$replyText = "เปิดบ้านแล้ว";
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function openRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			// $replyText[] = $locals['openMessage'];
			// $replyText[] = "เปิดรอบย่อยที่ #".$this->datas['lap'];
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				]
			];
			$this->datas['status'] = 'open';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'เปิดรอบแล้ว';
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'กรุณาสรุปผลให้เรียบร้อยก่อน';
		}
		return $replyText;
	}

	public function closeRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'open') {
			// $replyText[] = $locals['closeMessage'];
			// $replyText[] = 'ปิดรอบย่อยที่ #' . $this->datas['lap'];
			$checkAll = $this->checkAll($con);
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				$checkAll["messages"][0],
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			$this->datas['status'] = 'check';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'rollBack' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'ปิดรอบแล้ว';
		}
		return $replyText;
	}

	public function restRound(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultRest($con, $this->database, $this->datas['game'], $this->datas['bigLap'], $resetTime, $this->datas['startTime']);
			$bigLap = $this->datas['bigLap'];
			$replyText = 'ปิดรอบใหญ่ที่ #'.$bigLap;
			$this->datas['lap'] = 1;
			$this->datas['bigLap'] = $bigLap + 1;
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['count' => 0, 'wage' => 0, 'deposite' => 0])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else {
			$replyText = 'โปรดเช็คสถานะ';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function closeGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultClose($con, $this->database, $this->datas['game'], $resetTime);
			$replyText = 'ปิดบ้านแล้ว';
			$this->rebuildReport($con, $this->database);
			$this->datas['status'] = 'down';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'rollBack' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['tag' => 0, 'count' => 0, 'wage' => 0, 'deposite' => 0, 'rollBack' => null])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'ยังไม่ปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			$replyText = 'ยังไม่สรุป';
		} else {
			$replyText = 'ปิดบ้านแล้ว';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function saveResult(DatabaseInterface $con, $text)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] == 'down' || $this->datas['status'] == 'close') {
			return;
		}
		if ($this->datas['status'] != 'check') {
			return 'กรุณาปิดรอบก่อน';
		}
		$texts = str_split(substr($text, 1));
		if (count($texts) != 3) {
			$error = 1;
		}
		foreach ((array) $texts as $result)
		{
			if ($result>6 || $result<1 || !is_numeric($result)) {
				$error = 1;
			}
			$resultText .= " $result";
			$results[] = $result;
		}
		if ($error == 1) {
			$replyText = "สรุปผิด";
		} else {
			$replyText = "สรุป$resultText\r\n\r\nยันยืนผลสรุป yes หรือ @ok";
			sort($results);
			$result = json_encode($results);
			$con->startQuery()
				->update(['result' => $result])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function submit(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] != 'check') {
			return;
		}
		if (empty($this->results)) {
			return 'สรุปผิด';
		} else {
			$con->startQuery()
				->update(['rollBack' => null])
				->set($this->database)
				->build()
				->query()
				->close();
			$recieveDatabases = $con->startQuery()
				->select(['id', 'content'])
				->from($this->database)
				->where('content IS NOT NULL')
				->and('content != ""')
				->and('content != "{}"')
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			foreach ((array) $recieveDatabases as $rows)
			{
				$con->startQuery()
					->update(['rollBack' => str_replace("\\", "\\\\\\\\", $rows['content'])])
					->set($this->database)
					->where(['id' => $rows['id']])
					->build()
					->query()
					->close();
			}
			$con->startQuery()
				->update(['rollBack' => null])
				->set('money', 'money')
				->build()
				->query()
				->close();
			$replyText = $this->result($con, $this->database);
			$this->datas['lap']++;
			$this->datas['status'] = 'close';
			unset($this->results);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'result' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function setup(DatabaseInterface $con, $admins, $force)
	{
		if (!in_array($this->userId, $admins)) {
			return;
		}
		$database = $con->getDatabase();

		$existedDatabases = $con->startQuery()
			->select('SCHEMA_NAME')
			->from('SCHEMATA', 'information_schema')
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array($database, $existedDatabases)) {
			$con->startQuery()
				->create()
				->database($database)
				->build()
				->query()
				->close();
		}

		if (!in_array('money', $existedDatabases)) {
			$con->startQuery()
				->create()
				->database('money')
				->build()
				->query()
				->close();
		}

		$allTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => $database])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		$moneyTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => 'money'])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array('money', $moneyTables)) {
			$this->rebuildMoney($con);
			$install .= ' money';
		}
		if (!in_array('reportMoney', $moneyTables)) {
			$this->rebuildReportMoney($con);
			$install .= ' reportMoney';
		}
		if (!in_array('login', $allTables)) {
			$this->rebuildLogin($con);
			$install .= ' login';
		}
		if ($force === true || !in_array($this->database, $allTables)) {
			$this->rebuildHilo($con);
			$install .= ' hilo';
		}
		if ($force === true || !in_array('reportHilo', $allTables)) {
			$this->rebuildReport($con, $this->database);
			$install .= ' reportHilo';
		}
		if ($force === true || !in_array('reportAllHilo', $allTables)) {
			$this->rebuildAllReport($con, $this->database);
			$install .= ' reportAllHilo';
		}

		if (!empty($install)) {
			$replyText = "ติดตั้ง$install สำเร็จ";
		} else {
			$replyText = "มีฐานข้อมูลอยู่แล้ว";
		}
		return $replyText;
	}

	protected function rule()
	{
		if ((int) $this->datas['rule'] == 0) {
			for ($i=1; $i<=6; $i++)
			{
				$rewards[$i] = 1;
				for ($k=$j; $k<=6; $k++)
				{
					$rewards[$i.$j] = 5;
				}
			}
			$rewards[123] = 5;
			$rewards[456] = 5;
			$rewards[111] = 24;
			$rewards[222] = 24;
			$rewards[333] = 24;
			$rewards[444] = 24;
			$rewards[555] = 24;
			$rewards[666] = 24;
			$anotherRewards = [
				'ต'  => 1,
				'ส'  => 1,

				'1ต' => 2,
				'2ต' => 2,
				'3ต' => 3,
				'4ต' => 4,
				'5ต' => 5,
				'6ต' => 5,

				'ต1' => 2,
				'ต2' => 2,
				'ต3' => 3,
				'ต4' => 4,
				'ต5' => 5,
				'ต6' => 5,

				'6ส' => 2,
				'5ส' => 2,
				'4ส' => 3,
				'3ส' => 4,
				'2ส' => 5,
				'1ส' => 5,

				'ส6' => 2,
				'ส5' => 2,
				'ส4' => 3,
				'ส3' => 4,
				'ส2' => 5,
				'ส1' => 5,

				'h'  => 5,
			];
			foreach ((array) $anotherRewards as $key => $value)
			{
				$rewards[$key] = $value;
			}

			foreach ((array) $this->results as $result)
			{
				$results[] = $result;
				$sum += $result;
			}
			if ($sum>11) {
				$r = 'ส';
			} else if ($sum<11) {
				$r = 'ต';
			} else {
				$r = 'h';
			}
			sort($results);
			$threeResults = $results;
			foreach (array_count_values($threeResults) as $key => $value)
			{
				$rewards[$key] *= $value;
			}
			$results[] = $threeResults[0].$threeResults[1];
			$results[] = $threeResults[0].$threeResults[2];
			$results[] = $threeResults[1].$threeResults[2];
			$results[] = $threeResults[0].$threeResults[1].$threeResults[2];
			$results[] = $r;
			foreach ((array) $threeResults as $result)
			{
				$results[] = "{$r}{$result}";
				$results[] = "{$result}{$r}";
			}

			if (!in_array(123, $results) && (in_array(12, $results) || in_array(23, $results) || in_array(13, $results))) {
				$reduces[123] = 2;
			}
			if (!in_array(456, $results) && (in_array(45, $results) || in_array(56, $results) || in_array(46, $results))) {
				$reduces[456] = 2;
			}
		} else if ((int) $this->datas['rule'] == 1) {
			for ($i=1; $i<=6; $i++)
			{
				$rewards[$i] = 1;
				for ($j=$i; $j<=6; $j++)
				{
					$rewards[$i.$j] = 5;
					for ($k=$j; $k<=6; $k++)
					{
						$rewards[$i.$j.$k] = 5;
						$rewards[$i.$j.$k."สป"] = 25;
					}
				}
			}
			$rewards[11] = 6;
			$rewards[22] = 6;
			$rewards[33] = 6;
			$rewards[44] = 6;
			$rewards[55] = 6;
			$rewards[66] = 6;
			$rewards[111] = 20;
			$rewards[222] = 20;
			$rewards[333] = 20;
			$rewards[444] = 20;
			$rewards[555] = 20;
			$rewards[666] = 20;
			$anotherRewards = [
				'ต'  => 1,
				'ส'  => 1,

				'1ต' => 2,
				'2ต' => 2,
				'3ต' => 3,
				'4ต' => 4,
				'5ต' => 5,
				'6ต' => 5,

				'ต1' => 2,
				'ต2' => 2,
				'ต3' => 3,
				'ต4' => 4,
				'ต5' => 5,
				'ต6' => 5,

				'6ส' => 2,
				'5ส' => 2,
				'4ส' => 3,
				'3ส' => 4,
				'2ส' => 5,
				'1ส' => 5,

				'ส6' => 2,
				'ส5' => 2,
				'ส4' => 3,
				'ส3' => 4,
				'ส2' => 5,
				'ส1' => 5,

				'11ฮ'  => 6,
			];
			foreach ((array) $anotherRewards as $key => $value)
			{
				$rewards[$key] = $value;
			}

			foreach ((array) $this->results as $result)
			{
				$results[] = $result;
				$sum += $result;
			}
			if ($sum>11) {
				$r = 'ส';
			} else if ($sum<11) {
				$r = 'ต';
			} else {
				$r = '11ฮ';
			}
			sort($results);
			$threeResults = $results;
			foreach (array_count_values($threeResults) as $key => $value)
			{
				$rewards[$key] *= $value;
			}
			$results[] = $threeResults[0].$threeResults[1];
			$results[] = $threeResults[0].$threeResults[2];
			$results[] = $threeResults[1].$threeResults[2];
			$results[] = $threeResults[0].$threeResults[1].$threeResults[2];
			$results[] = $threeResults[0].$threeResults[1].$threeResults[2]."สป";
			$results[] = $r;
			foreach ((array) $threeResults as $result)
			{
				$results[] = "{$r}{$result}";
				$results[] = "{$result}{$r}";
			}

			for ($i=1; $i<=6; $i++)
			{
				$reduces[$i.$threeResults[0].$threeResults[1]] = 1;
				$reduces[$i.$threeResults[0].$threeResults[1]."สป"] = -1;
				$reduces[$i.$threeResults[1].$threeResults[2]] = 1;
				$reduces[$i.$threeResults[1].$threeResults[2]."สป"] = -1;
				$reduces[$i.$threeResults[0].$threeResults[2]] = 1;
				$reduces[$i.$threeResults[0].$threeResults[2]."สป"] = -1;

				$reduces[$threeResults[0].$i.$threeResults[1]] = 1;
				$reduces[$threeResults[0].$i.$threeResults[1]."สป"] = -1;
				$reduces[$threeResults[1].$i.$threeResults[2]] = 1;
				$reduces[$threeResults[1].$i.$threeResults[2]."สป"] = -1;
				$reduces[$threeResults[0].$i.$threeResults[2]] = 1;
				$reduces[$threeResults[0].$i.$threeResults[2]."สป"] = -1;

				$reduces[$threeResults[0].$threeResults[1].$i] = 1;
				$reduces[$threeResults[0].$threeResults[1].$i."สป"] = -1;
				$reduces[$threeResults[1].$threeResults[2].$i] = 1;
				$reduces[$threeResults[1].$threeResults[2].$i."สป"] = -1;
				$reduces[$threeResults[0].$threeResults[2].$i] = 1;
				$reduces[$threeResults[0].$threeResults[2].$i."สป"] = -1;
			}
		}
		return [$rewards, $results, $reduces];
	}
}