<?php
namespace Koob\IncomeOutcomeBundle\Model;

use Koob\InterfaceBundle\IncomeOutcomeInterface;
use Koob\InterfaceBundle\DatabaseInterface;

Class FunctionModel implements IncomeOutcomeInterface
{
	protected $func;

	public function rebuildTable(DatabaseInterface $con) {
		$con->startQuery()
			->drop()
			->table('incomeoutcome')
			->build()
			->query()
			->close();

		$con->startQuery()
			->create([
				'content'       => 'VARCHAR(255)',
				'incomeoutcome' => 'VARCHAR(255)',
				'time'          => 'VARCHAR(255)',
				'date'          => 'VARCHAR(255)',
				'month'         => 'VARCHAR(255)',
				'year'          => 'VARCHAR(255)',
			])
			->table('incomeoutcome')
			->build()
			->query()
			->close();
	}

	public function check(DatabaseInterface $con, $date, $month, $year)
	{
		$resultRows = $con->startQuery()
			->select(['content', 'incomeoutcome', 'time'])
			->from('incomeoutcome')
			->where(['date' => $date, 'month' => $month, 'year' => $year])
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$id = 1;
		foreach ((array) $resultRows as $rows) {
			$reply .= "\r\n".$id.') '.$rows['time'].' | '.$rows['content'].' = '.$rows['incomeoutcome'];
			$money += $rows['incomeoutcome'];
			$id ++;
		}
		$reply = "สรุปยอดวันที่ {$date}/{$month}/{$year}" . $reply . "\r\nรวม ".(int) $money." บาท";
		return $reply;
	}

	public function checkAll(DatabaseInterface $con)
	{
		$resultRows = $con->startQuery()
			->select(['incomeoutcome'])
			->from('incomeoutcome')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		foreach ((array) $resultRows as $rows)
		{
			if ($rows['incomeoutcome'] > 0) {
				$deposite += $rows['incomeoutcome'];
			} else {
				$withdraw += $rows['incomeoutcome'];
			}
			$money += $rows['incomeoutcome'];
		}
		$reply = '----สรุปยอด----';
		$reply .= "\n".sprintf('ฝาก %.2f บาท', $deposite);
		$reply .= "\n".sprintf('ถอน %.2f บาท', $withdraw);
		$reply .= "\n".sprintf('คงเหลือ %.2f บาท', $money);
		$month = date('n');
		$hour = date('H');
		$minute = date('i');
		if (in_array($month, [1, 3, 5, 7, 8, 10, 12])) {
			$lastDay = 31;
		} else if (in_array($month, [4, 6, 9, 11])) {
			$lastDay = 30;
		} else {
			$lastDay = 28;
		}
		$day = $hour >= 19 && $minute >= 30 ? date('j') : date('j')-1;
		$remainingDay = $lastDay - $day;
		$average = ($remainingDay == 0) ? $money : $money/$remainingDay;
		$reply .= "\r\n".sprintf('เฉลี่ย %d วันวันละ %d บาท', $remainingDay, $average);
		return $reply;
	}

	public function insert(DatabaseInterface $con, $contents)
	{
		$money = $contents[0];
		$content = $contents[1];
		unset($contents[0], $contents[1]);
		foreach ((array) $contents as $text)
		{
			$content .= empty($text) ? '' : " $text";
		}
		$time = date('H:i:s');
		$date = date('j');
		$month = date('n');
		$year = date('Y');
		$con->startQuery()
			->insert([
				'content' 		=> $content,
				'incomeoutcome' => $money,
				'time' 			=> $time,
				'date' 			=> $date,
				'month' 		=> $month,
				'year' 			=> $year,
			])
			->into('incomeoutcome')
			->build();
		$replyText = $con->query(true) ? "บันทึก $content สำเร็จ" : $replyText = 'บันทึกไม่สำเร็จ';
		$con->close();
		return $replyText;
	}

	public function fixContent(DatabaseInterface $con, $id, $contents)
	{
		$content = $contents[2];
		unset($contents[0], $contents[1], $contents[2]);
		foreach ((array) $contents as $text)
		{
			$content .= empty($text) ? '' : " $text";
		}

		[$id, $date, $month, $year] = explode(':', $id);
		$id--;
		$date = empty($date) ? date('j') : $date;
		$month = empty($month) ? date('n') : $month;
		$year = empty($year) ? date('Y') : $year;
		$resultRows = $con->startQuery()
			->select(['id', 'content'])
			->from('incomeoutcome')
			->where(['date' => $date, 'month' => $month, 'year' => $year])
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$targetId = $resultRows[0]['id']+$id;

		$con->startQuery()
			->update(['content' => $content])
			->set('incomeoutcome')
			->where(['id' => $targetId])
			->build();
		$replyText = $con->query(true) ? "แก้ไขข้อความ ".$resultRows[$id]['content']." เป็น $content" : "แก้ไขไม่สำเร็จ";
		$con->close();
		if (empty($resultRows[$id]['content'])) {
			$replyText = 'id ไม่ถูกต้อง';
		}
		return $replyText;
	}

	public function fixMoney(DatabaseInterface $con, $id, $money)
	{
		[$id, $date, $month, $year] = explode(':', $id);
		$id--;
		$date = empty($date) ? date('j') : $date;
		$month = empty($month) ? date('n') : $month;
		$year = empty($year) ? date('Y') : $year;
		$resultRows = $con->startQuery()
			->select(['id', 'incomeoutcome'])
			->from('incomeoutcome')
			->where(['date' => $date, 'month' => $month, 'year' => $year])
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$targetId = $resultRows[0]['id']+$id;

		$con->startQuery()
			->update(['incomeoutcome' => $money])
			->set('incomeoutcome')
			->where(['id' => $targetId])
			->build();
		$replyText = $con->query(true) ? "แก้ไขยอด ".$resultRows[$id]['incomeoutcome']." เป็น $money" : "แก้ไขไม่สำเร็จ";
		$con->close();
		if (empty($resultRows[$id]['incomeoutcome'])) {
			$replyText = 'id ไม่ถูกต้อง';
		}
		return $replyText;
	}

	public function setup(DatabaseInterface $con, $userId, $admins, $force = false)
	{
		if (!in_array($userId, $admins)) {
			return;
		}
		$database = $con->getDatabase();

		$existedDatabases = $result = $con->startQuery()
			->select('SCHEMA_NAME')
			->from('SCHEMATA', 'information_schema')
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array($database, $existedDatabases)) {
			$con->startQuery()
				->create()
				->database($database)
				->build()
				->query()
				->close();
		}

		$allTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => $database])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if ($force === true || !in_array('incomeoutcome', $allTables)) {
			$this->rebuildTable($con);
			$replyText = ' incomeoutcome';
		}

		if (isset($replyText)) {
			$replyText = 'ติดตั้ง'.$replyText.' สำเร็จ';
		} else {
			$replyText = 'มีฐานข้อมูลอยู่แล้ว';
		}
		return $replyText;
	}
}