<?php
namespace Koob\LineApiBundle\Model;

use Koob\InterfaceBundle\LineApiInterface;
use Koob\LocalBundle\Controller\LocalController;

class LineApiModel implements LineApiInterface
{
	/**
	* @var access token of line message api
	*/
	protected $accessToken;

	protected $url;

	protected $ch;

	protected $destination;

	protected $ignores;

	protected $messages;

	protected $type;

	protected $roomId;

	protected $groupId;

	protected $userId;

	protected $method;

	protected $local;

	protected $errorLocation = __DIR__.'/../../../logs/line_api_error_logs.log';

	function __construct()
	{
		$this->local = new LocalController;
	}

	public function startApi()
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$this->ch = $ch;
	}

	public function setAccessToken($accessToken)
	{
		$this->accessToken = $accessToken;
	}

	public function setDestination($destination)
	{
		if (is_array($destination)) {
			$this->destination = $destination;
		} else {
			$this->destination = [$destination];
		}
	}

	public function setIgnore($ignores)
	{
		$this->ignores = $ignores;
	}

	public function setMessage($messages, $isRaw = false)
	{
		unset($this->messages);
		$this->messages = [];
		if (!$isRaw) {
			if (is_array($messages) && sizeof($messages) == 1) {
				$isOver = true;
				$texts[0] = $messages[0];
			} else if (!is_array($messages) && stripos($messages, "\r\n")) {
				$isOver = true;
				$texts[0] = $messages;
			} else {
				$texts = $messages;
				$isOver = false;
			}
			for ($i=0; $isOver && mb_strlen($texts[$i]) > 1500; $i++)
			{
				$temp = 0;
				while ($temp < 1500) {
					$pos = $temp;
					$temp = mb_stripos($texts[$i], "\r\n", $temp+1);
				}
				$texts[$i+1] = mb_substr($texts[$i], $pos);
				$texts[$i] = mb_substr($texts[$i], 0, $pos);
			}
			$messages = $texts;
			if (is_array($messages)) {
				foreach ((array) $messages as $message)
				{
					$messages = [
						'type' => 'text',
						'text' => $message,
					];
					$this->messages[] = $messages;
				}
			} else {
				$this->messages = [
					[
						'type' => 'text',
						'text' => $messages,
					]
				];
			}
		} else {
			$this->messages = $messages;
		}
	}

	public function setUrl($url)
	{
		$this->url = $url;
	}

	public function setType($type)
	{
		$this->type = $type;
	}

	public function setMethod($method)
	{
		switch ($method)
		{
			case 'push':
				$this->setUrl('https://api.line.me/v2/bot/message/push');
				$this->setType('to');
				break;

			case 'reply':
				$this->setUrl('https://api.line.me/v2/bot/message/reply');
				$this->setType('replyToken');
				break;

			case 'getByuserId':
				$this->setUrl('https://api.line.me/v2/bot/profile/');
				break;

			case 'getByRoomId':
				$this->setUrl('https://api.line.me/v2/bot/room/'.$this->roomId.'/member/');
				break;

			case 'getByGroupId':
				$this->setUrl('https://api.line.me/v2/bot/group/'.$this->groupId.'/member/');
				break;

			default:
				$this->setUrl(null);
				$this->setType(null);
				$this->errorLog("type $method not found.");
				break;
		}
		$this->method = $method;
	}

	public function setOption($optionName, $option)
	{
		$optionName = strtoupper($optionName);
		curl_setopt($this->ch, $optionName, $option);
	}

	public function setRoomId($roomId)
	{
		$this->roomId = $roomId;
	}

	public function setGroupId($groupId)
	{
		$this->groupId = $groupId;
	}

	public function send()
	{
		$useNotify = false;
		foreach ((array) $this->destination as $destination) {
			if ($useNotify) {
				break;
			}
			$datas = [
				$this->type => $destination,
				'messages' => $this->messages,
			];
			$headers = array('Content-Type: application/json', 'Authorization: Bearer ' . $this->accessToken);
			$post = json_encode($datas);
			curl_setopt($this->ch, CURLOPT_URL, $this->url);
			curl_setopt($this->ch, CURLOPT_POSTFIELDS, $post);
			curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, "POST");
			$return = curl_exec($this->ch);
			if ($return == '{"message":"You have reached your monthly limit."}') {
				$useNotify = true;
				$notifys = $this->local->getParameter('notifys');
				$playRooms = $notifys['playRoom'];
				$reportRooms = $notifys['reportRoom'];
				$withdrawRoom = $notifys['withdrawRoom'];
				$checkPlayRoom = $this->local->getParameter('playRoom');
				$checkReportRoom = $this->local->getParameter('reportRoom');
				foreach ((array) $this->ignores as $ignore)
				{
					$id = array_search($ignore, $checkPlayRoom);
					if (!empty($id)) {
						$ignoreIds[] = $id;
					}
				}
				if (in_array($destination, (array) $checkPlayRoom) || $destination == $checkPlayRoom) {
					foreach ((array) $playRooms as $id => $token) {
						if (in_array($id, (array) $ignoreIds)) {
							continue;
						}
						$this->sendNotify($token, $this->messages[0]['text']);
					}
				} else if ($destination == $checkReportRoom) {
					foreach ((array) $reportRooms as $token) {
						$this->sendNotify($token, $this->messages[0]['text']);
					}
				} else {
					foreach ((array) $withdrawRoom as $token) {
						$this->sendNotify($token, $this->messages[0]['text']);
					}
				}
			} else if ($return != '{}') {
				$this->errorLog($return);
			}
		}
	}

	public function get($userId)
	{
		$headers = array('Authorization: Bearer ' . $this->accessToken);
		curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
		$this->setMethod('getByuserId');
		curl_setopt($this->ch, CURLOPT_URL, $this->url.$userId);
		$result = curl_exec($this->ch);
		$reply = json_decode($result, 1);
		if (empty($reply['userId']) && !empty($this->roomId)) {
			$this->setMethod('getByRoomId');
			curl_setopt($this->ch, CURLOPT_URL, $this->url.$userId);
			$result = curl_exec($this->ch);
			$reply = json_decode($result, 1);
		}
		if (empty($reply['userId']) && !empty($this->groupId)) {
			$this->setMethod('getByGroupId');
			curl_setopt($this->ch, CURLOPT_URL, $this->url.$userId);
			$result = curl_exec($this->ch);
			$reply = json_decode($result, 1);
		}
		return $reply;
	}

	public function sendNotify($token, $message, $count = 0)
	{
		$message = str_replace('%', '%25', $message);
		$message = str_replace('+', '%2B', $message);
		$message = str_replace('&', '%26', $message);
		$post = "message=$message";
		$headers = array('Content-type: application/x-www-form-urlencoded', 'Authorization: Bearer ' . $token);
		curl_setopt($this->ch, CURLOPT_URL, 'https://notify-api.line.me/api/notify');
		curl_setopt($this->ch, CURLOPT_POSTFIELDS, $post);
		curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, "POST");
		$return = curl_exec($this->ch);
		if (!in_array($return, ['{}', '{"status":200,"message":"ok"}'])) {
			$this->errorLog($return);
		}
	}

	public function close()
	{
		curl_close($this->ch);
	}

	private function errorLog($error)
	{
		$error = empty($error) ? 'No error message.' : $error;
		error_log('{'.date("Y-m-d H:i:s").'} '.$error."\n", 3, $this->errorLocation);
	}
}