<?php
namespace Koob\MoneyBundle\Model;

use Koob\InterfaceBundle\DatabaseInterface;
use Koob\GameBundle\Model\GameModel;

class FunctionModel extends GameModel
{
	public function getData(DatabaseInterface $con, $table = null)
	{
		$resultRows = $con->startQuery()
			->select(['id', 'net', 'admin'])
			->from('money')
			->where(['userId' => $this->userId])
			->build()
			->query()
			->fetchAssoc();

		$this->id = $resultRows[0]['id'];
		$this->net = $resultRows[0]['net'];
		$this->admin = $resultRows[0]['admin'];

		return $this->id;
	}

	public function insert(DatabaseInterface $con, $table, $extraInserts = [])
	{
		$con->startQuery()
			->insert([
				'name'    => $this->codeName($this->name),
				'userId'  => $this->userId,
				'net'     => 0,
			])
			->into('money', 'money')
			->build()
			->query()
			->close();

		$resultRows = $con->startQuery()
			->select(['id'])
			->from($table)
			->where(['userId' => $this->userId])
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$this->id = $resultRows[0]['MAX(id)'];
	}

	public function fixMoney(DatabaseInterface $con, $text)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		$operator = stripos($text, '+') ? '+' : '-';
		$explodeTexts = explode($operator, $text);
		$memberId = substr($explodeTexts[0], 1);
		$changeNet = $explodeTexts[1];

		$receiveRows = $con->startQuery()
			->select(['userId', 'name'])
			->from('money')
			->where(['id' => $memberId])
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$memberUserId = $receiveRows[0]['userId'];
		$memberName = json_decode($receiveRows[0]['name'], 1);

		if (!empty($memberUserId)) {
			$receiveRows = $con->startQuery()
				->select(['net'])
				->from('money')
				->where(['userId' => $memberUserId])
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			$memberNet = $receiveRows[0]['net'];

			if ($operator == '+') {
				$prefix = 'เพิ่ม';
				$remaining = $memberNet + $changeNet;
				$fixNet = "+$changeNet";
				if (!empty($explodeTexts[2])) {
					$percent = substr($explodeTexts[2], 0, strlen($explodeTexts[2]));
					$bonus = number_format($changeNet*$percent/100, 0, '.', '');
					$remaining += (int) $bonus;
					$bonusText = "โบนัส+$bonus ";
				}
			} else {
				$prefix = 'ลด';
				$remaining = $memberNet - $changeNet;
				$fixNet = "-$changeNet";
			}
			$con->startQuery()
				->update(['net' => $remaining])
				->set('money', 'money')
				->where(['userId' => $memberUserId])
				->build()
				->query()
				->close();
			$replyText = "{$prefix}เครดิต $memberName {$operator}{$changeNet} $bonusText= " . $remaining;
		} else {
			$replyText = 'โปรดระบุเลข id ให้ถูกต้อง';
		}
		return $replyText;
	}

	public function setup(DatabaseInterface $con, $userId, $admins, $force = false)
	{
		if (!in_array($userId, $admins)) {
			return;
		}

		$existedDatabases = $result = $con->startQuery()
			->select('SCHEMA_NAME')
			->from('SCHEMATA', 'information_schema')
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array('money', $existedDatabases)) {
			$con->startQuery()
				->create()
				->database('money')
				->build()
				->query()
				->close();
		}

		$moneyTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => 'money'])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array('money', $moneyTables)) {
			$this->rebuildMoney($con);
			$install .= ' money';
		}
		if (!in_array('reportMoney', $moneyTables)) {
			$this->rebuildReportMoney($con);
			$install .= ' reportMoney';
		}

		if (!empty($install)) {
			$replyText = "ติดตั้ง$install สำเร็จ";
		} else {
			$replyText = "มีฐานข้อมูลอยู่แล้ว";
		}
		return $replyText;
	}
}