<?php
namespace Koob\MoneyBundle\Model;

use Koob\InterfaceBundle\LobbyInterface;
use Koob\DatabaseBundle\Controller\DatabaseController;
use Koob\LineApiBundle\Controller\LineApiController;
use Koob\LocalBundle\Controller\LocalController;
use Koob\MoneyBundle\Controller\FunctionController;

class LobbyModel implements LobbyInterface
{
	protected $con;

	protected $api;

	protected $func;

	protected $local;

	function __construct()
	{
		$this->con = new DatabaseController;
		$this->api = new LineApiController;
		$this->func = new FunctionController;
		$this->local = new LocalController;

		$this->con->setHost($this->local->getParameter('host'))
			->setUser($this->local->getParameter('user'))
			->setPass($this->local->getParameter('pass'))
			->setDatabase('money');

		$this->api->setAccessToken($this->local->getParameter('accessToken'));
	}

	public function run($content)
	{
		if (!empty($content['events'])) {
			foreach ((array) $content['events'] as $events)
			{
				$admins           = $this->local->getParameter('admins');
				$forceRebuild     = $this->local->getParameter('forceRebuild');
				$allowGroups      = $this->local->getParameter('allowGroups');
				$ignoreUserIds    = $this->local->getParameter('ignoreUserIds');
				$autoReplyMessage = $this->local->getParameter('gameMessages')['autoReplyMessage'];

				$groupId = $events['source']['groupId'];

				if (in_array($groupId, $allowGroups)) {
					$type = $events['message']['type'];
					$originalText = $events['message']['text'];
					$userId = $events['source']['userId'];
					$roomId = $events['source']['roomId'];

					$ids = [
						'userId'  => $userId,
						'roomId'  => $roomId,
						'groupId' => $groupId,
					];
					$userId = $this->func->getApi($this->api, $ids);

					$id = $this->func->getData($this->con, 'money');

					$text = str_replace([' ', "\r\n"], '', $originalText);
					$explodeTexts = explode(' ', $originalText);

					if (empty($id) && !empty($userId)) {
						$id = $this->func->insert($this->con, 'money');
					}

					if (in_array($text, ['ox', 'Ox'])) {
						$replyText = "id:$id\r\nuserId:$userId";
					}

					if ($text == 'ALL') {
						$replyText = "groupId:$groupId\r\nRoomId:$roomId\r\nuserId:$userId";
					}

					if ($type == 'image' || substr($text, 0, 3) == '@id') {
						$replyText = $this->func->checkMoney($this->con, 'money', $ignoreUserIds);
					}

					if ($text[0] == '$'){
						$replyText = $this->func->fixMoney($this->con, $text);
					}

					if (strcasecmp($explodeTexts[0], 'setadmin') == 0) {
						$replyText = $this->func->setAdmin($this->con, 'money', $admins, $explodeTexts[1], $explodeTexts[2]);
					}

					if (strcasecmp($explodeTexts[0], 'checkid') == 0) {
						$replyText = $this->func->checkId($this->con, 'money', $admins, $explodeTexts);
					}

					if (strcasecmp($text, 'setup') == 0) {
						$replyText = $this->func->setup($this->con, $userId, $admins, $forceRebuild);
					}
				} else {
					$replyText = $autoReplyMessage."\r\ngroupId : $groupId";
				}
				return $replyText;
			}
		}
	}
}