<?php
namespace Koob\PokeBundle\Model;

use Koob\InterfaceBundle\PokeInterface;
use Koob\InterfaceBundle\DatabaseInterface;
use Koob\GameBundle\Model\GameModel;

class FunctionModel extends GameModel implements PokeInterface
{
	protected $database = 'poke';

	protected $attrs = ['player', 'min', 'max', 'reply', 'limit', 'multi', 'maxWage', 'deli', 'allDeli', 'depo', 'allDepo', 'recc', 'cp', '$', 'deng'];

	public function rebuildPoke(DatabaseInterface $con)
	{
		$con->startQuery()
			->drop()
			->table($this->database)
			->build()
			->query()
			->close();

		$con->startQuery()
			->create([
				'name'     => 'VARCHAR(255)',
				'userId'   => 'VARCHAR(255) UNIQUE',
				'admin'    => 'VARCHAR(255)',
				'content'  => 'TEXT',
				'result'   => 'TEXT',
				'rollBack' => 'TEXT',
			])
			->table($this->database)
			->build()
			->query()
			->close();

		$datas = [
			'status' => 'down',
			'game'    => 0,
			'bigLap'  => 1,
			'lap'     => 1,
			'player'  => 4,
			'min'     => 20,
			'max'     => 200,
			'multi'   => 'yes',
			'reply'   => 'yes',
			'limit'   => 1000,
			'maxWage' => 'no',
			'depo'    => 'no',
			'deng'    => 2,
		];
		$data = json_encode($datas);
		$con->startQuery()
			->insert([
				'name' => '"index"',
				'userId' => 'index',
				'content' => $data,
			])
			->into($this->database)
			->build()
			->query()
			->close();
	}

	public function wage(DatabaseInterface $con, $text)
	{
		$hasDeli = stripos($text, $this->datas['deli']);
		if ($this->datas['status'] == 'down') {
			return;
		} else if ($this->datas['status'] == 'close') {
			return 'ยังไม่เปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			return 'แทงไม่ทัน';
		} else if (!$hasDeli) {
			return "รูปแบบการแทง xx".$this->datas['deli']."xxx";
		}
		if ($this->datas['multi'] == 'no') {
			$this->contents['multi'] = 'yes';
		}
		if ($this->net >= $this->datas['limit'] && $this->contents['multi'] != 'yes' && $this->datas['multi'] == 'yes') {
			$this->contents['multi'] = 'yes';
			$replyTexts[] = "เครดิตเกิน ".$this->datas['limit']." บังคับเล่นเด้ง";
		}
		$explodeTexts = explode($this->datas['allDeli'], $text);
		$oldContents = $this->contents;
		$contentmulti = $this->contents['multi'];
		unset($this->contents['multi']);
		foreach ((array) $explodeTexts as $wageList)
		{
			[$player, $wage] = explode($this->datas['deli'], $wageList);
			if (!is_numeric($player)) {
				$errors[] = "การแทง $wageList ขาไม่ถูกต้อง มีขา 1-".$this->datas['player'].' เท่านั้น❗';
				continue;
			}
			if (!is_numeric($wage)) {
				$errors[] = "การแทง $wageList ไม่ถูกต้อง❗";
				continue;
			}
			if ($wage < $this->datas['min'] || $wage > $this->datas['max']) {
				$errors[] = "การแทง $wageList ไม่ถูกต้อง วงเงินการแทง ".$this->datas['min'].'-'.$this->datas['max'].' บาท❗';
				continue;
			}
			if ($wage < $this->contents['maxWage'] && $this->datas['maxWage'] == 'yes') {
				$errors[] = "การแทง $wageList ไม่ถูกต้อง การแทงต้องมากกว่ารอบที่ผ่านมา หรือมากว่า ".$this->contents['maxWage']." บาท";
			}
			if (!empty($this->datas['depo']) && !empty($this->bonus) && $this->datas['depo'] > 0) {
				$max = $this->deposite*$this->datas['depo']/100;
				$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
				if ($wage < $maximum) {
					$errors[] = "แทง ".$this->datas['depo']."% ของยอดฝาก($max ขึ้นไป)";
					continue;
				}
			}
			if (!empty($this->datas['cp']) && $this->play != 'playing' && $this->checkPeople($con, $this->database, true) >= $this->datas['cp']) {
				$errors[] = "จำนวนคนครบ ".$this->datas['cp']." คนแล้ว";
				continue;
			}
			if (0 < $player && $player <= $this->datas['player']) {
				$this->contents[$player] = $wage;
			} else if ($player > 10) {
				$players = str_split($player);
				foreach ((array) $players as $playerNumber)
				{
					if (0 < $playerNumber && $playerNumber <= $this->datas['player']) {
						$this->contents[$playerNumber] = $wage;
					} else {
						$error .= "$playerNumber";
					}
				}
				if (!empty($error)) {
					$errors[] = "การแทง $error ขาไม่ถูกต้อง มีขา 1-".$this->datas['player'].' เท่านั้น❗';
					continue;
				}
			} else {
				$errors[] = "การแทง $wageList ขาไม่ถูกต้อง มีขา 1-".$this->datas['player'].' เท่านั้น❗';
				continue;
			}
		}
		foreach ((array) $this->contents as $key => $wage)
		{
			$allWageNet += is_numeric($key) ? (int) $wage : 0;
		}
		if (!empty($contentmulti)) {
			$this->contents['multi'] = $contentmulti;
		}
		if (!empty($this->datas['allDepo']) && !empty($this->bonus) && $this->datas['allDepo'] > 0)  {
			$max = $this->deposite*$this->datas['allDepo']/100;
			$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
			if ($allWageNet < $maximum) {
				$errors[] = "แทงทั้งหมด ".$this->datas['allDepo']."% ของยอดฝาก($max ขึ้นไป)";
			}
		}
		$allWageNet *= $this->contents['multi'] == 'yes' ? $this->datas['deng'] : 1;
		if ($allWageNet <= $this->net+$this->bonus) {
			$errorText = implode(" ", (array) $errors);
		} else if ($this->contents['multi'] == 'yes') {
			$errorText = "เครดิตไม่พอต่อการแทงเด้ง❗";
		} else {
			$errorText = "เครดิตไม่เพียงพอ❗";
		}
		$clearErrorText = str_replace([' ', "\r\n"], '', $errorText);
		if (empty($clearErrorText)) {
			$content = json_encode($this->contents);
			$con->startQuery()
				->update(['content' => $content, 'result' => 'playing'])
				->set($this->database)
				->where(['userId' => $this->userId])
				->build()
				->query()
				->close();
		}
		$replyTexts = $this->check();
		if ($this->datas['reply'] == 'yes') {
			if (!empty($errorText)) {
				$replyTexts .= "\r\n\r\n$errorText";
			}
		} else if ($this->datas['reply'] == 'flex') {
			if (!empty($clearErrorText)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							$errorText,
							'separator #000000',
						],
					],
				], 'xs');
			}
		} else {
			unset($replyTexts);
		}
		return $replyTexts;
	}

	public function unWage(DatabaseInterface $con, $text)
	{
		if ($this->datas['status'] != 'open') {
			return $this->datas['status'] == 'down' ? '' : "$this->name ปิดรอบแล้ว❗";
		}
		foreach ((array) $this->contents as $type => $wage)
		{
			unset($this->contents[$type]);
			$cancel .= " $type";
		}
		$content = $this->codename($this->contents);
		$con->startQuery()
			->update(['content' => $content, 'result' => null])
			->set($this->database)
			->where(['userId' => $this->userId])
			->build()
			->query()
			->close();
		if (!empty($cancel)) {
			if ($this->datas['reply'] == 'flex') {
				$replyTexts = $this->flexMessage([
					'old' => $this->check(),
					'body' => [
						[
							"ยกเลิกขา$cancel",
							"separator #000000",
						],
					],
				], 'xs');
			} else if ($this->datas['reply'] == 'yes') {
				$replyTexts = $this->check()."\r\n\r\nยกเลิกขา$cancel";
			}
		}
		return $replyTexts;
	}

	public function unWageAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		$con->startQuery()
			->update(['content' => '{}', 'result' => null])
			->set($this->database)
			->where('id>1')
			->build()
			->query()
			->close();
		return "ยกเลิกการแทงทั้งหมด";
	}

	public function domulti(DatabaseInterface $con)
	{
		if ($this->datas['multi'] != 'yes')
		{
			return;
		}
		if (in_array($this->datas['status'], ['open', 'close'])) {
			$contents = $this->contents;
			$multi = $contents['multi'];
			$maxWage = $contents['maxWage'];
			unset($contents['multi'], $contents['maxWage']);
			foreach ((array) $contents as $money)
			{
				$price += $money;
			}
			if ($this->net >= ($price * $this->datas['deng'])) {
				$this->contents['multi'] = 'yes';
				$content = json_encode($this->contents);
				$con->startQuery()
					->update(['content' => $content])
					->set($this->database)
					->where(['userId' => $this->userId])
					->build()
					->query()
					->close();
				if ($this->datas['reply'] == 'yes') {
					$replyText = "$this->name เล่นเด้ง";
				}
			} else {
				$replyText = "$this->name เครดิตไม่พอต่อการเล่นเด้ง❗";
			}
		} else if ($this->datas['status'] != 'down') {
			$replyText = "$this->name ปิดรอบแล้ว❗";
		}
		return $replyText;
	}

	public function doUnmulti(DatabaseInterface $con)
	{
		if ($this->datas['multi'] != 'yes')
		{
			return;
		}
		if ($this->net >= $this->datas['limit']) {
			$replyText = "$this->name เครดิตเกิน ".$this->datas['limit']." บังคับเล่นเด้ง";
		} else if (in_array($this->datas['status'], ['open', 'close'])) {
			unset($this->contents['multi']);
			$content = json_encode($this->contents);
			$con->startQuery()
					->update(['content' => $content])
					->set($this->database)
					->where(['userId' => $this->userId])
					->build()
					->query()
					->close();
			if ($this->datas['reply'] == 'yes') {
				$replyText = "$this->name ไม่เล่นเด้ง";
			}
		} else if ($this->datas['status'] != 'down') {
			$replyText = "$this->name ปิดรอบแล้ว❗";
		}
		return $replyText;
	}

	public function check()
	{
		if ($this->datas['multi'] == 'yes') {
			$suffix = $this->contents['multi'] == 'yes' ? ' ✔️' : ' ❌';
		}
		$replyTexts = $this->flexMessage([
			'altText' => "สรุปการแทงของ $this->name",
			'footer' =>	"{$this->net}💸{$this->bonus}{$suffix}",
			'body' => [
				[
					$this->name,
				],
			],
		], 'xs');
		$isEmpty = 1;
		$text = "$this->name แทง";
		foreach ((array) $this->contents as $type => $wageNet)
		{
			if (in_array($type, ['multi', 'maxWage'])) {
				continue;
			}
			$isEmpty = 0;
			$text .= " $type=$wageNet";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						$type,
						$wageNet,
					],
				],
			], 'xs');
		}
		if ($isEmpty) {
			$text .= "ไม่ได้แทง";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						'ไม่ได้แทง',
					],
				],
			], 'xs');
		}
		$text .= "\r\n{$this->net}💸{$this->bonus}";
		$replyTexts = $this->flexMessage([
			'old' => $replyTexts,
			'body' => [
				[
					'separator #000000',
				],
			],
		]);
		if ($this->datas['reply'] == 'yes') {
			$replyTexts = $text;
		}
		return $replyTexts;
	}

	public function checkAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}

		$resultRows = $con->startQuery()
			->select(['x.name, x.content, m.userId', 'm.net', 'm.bonus'])
			->from($this->database.' x, money.money m')
			->where('x.userId = m.userId')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$replyTexts = $this->flexMessage([
			'altText' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
			'header' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
		]);
		foreach ((array) $resultRows as $rows)
		{
			$contents = json_decode($rows['content'], 1);
			$name = json_decode($rows['name'], 1);
			if ($this->datas['multi'] == 'yes') {
				$suffix = $contents['multi'] == 'yes' ? ' ✔️' : ' ❌';
			}
			foreach ((array) $contents as $type => $wageNet)
			{
				if (in_array($type, ['multi', 'maxWage'])) {
					continue;
				}
				$reply .= " $type=$wageNet";
			}
			if ($rows['bonus'] > 0) {
				$bonusText = "💸".$rows['bonus']."=".($rows['bonus']+$rows['net']);
			}
			if (!empty($reply)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							"$name แทง$reply 💰".$rows['net']."{$bonusText}{$suffix}",
						],
					],
				]);
			}
			unset($reply, $bonusText);
		}
		$noPlayerReplys = [
			"type" => "text",
			"text" => "ยังไม่มีผู้เล่นแทง",
		];
		if (empty($replyTexts['messages'][0]['contents']['contents'][0]['body'])) {
			$replyTexts = [
				'messages' => [$noPlayerReplys],
				'raw' => true
			];
		}
		return $replyTexts;
	}

	public function openGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'down') {
			$sendText = $this->resultOpen($con, $this->database, $this->datas['game'], $resetTime);
			$game = $this->datas['game'] + 1;
			$replyText = "เริ่มเกมที่ #".$game;
			$this->datas['status'] = 'close';
			$this->datas['game'] = $game;
			$this->datas['bigLap'] = 1;
			$this->datas['lap'] = 1;
			$this->datas['startTime'] = date('H.i');
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else {
			$replyText = "เปิดบ้านแล้ว";
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function openRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			// $replyText[] = $locals['openMessage'];
			// $replyText[] = "เปิดรอบย่อยที่ #".$this->datas['lap'];
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			$this->datas['status'] = 'open';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'เปิดรอบแล้ว';
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'กรุณาสรุปผลให้เรียบร้อยก่อน';
		}
		return $replyText;
	}

	public function closeRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'open') {
			// $replyText[] = $locals['closeMessage'];
			// $replyText[] = 'ปิดรอบย่อยที่ #' . $this->datas['lap'];
			$checkAll = $this->checkAll($con);
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				$checkAll["messages"][0],
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			$this->datas['status'] = 'check';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'rollBack' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'ปิดรอบแล้ว';
		}
		return $replyText;
	}

	public function restRound(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultRest($con, $this->database, $this->datas['game'], $this->datas['bigLap'], $resetTime, $this->datas['startTime']);
			$bigLap = $this->datas['bigLap'];
			$replyText = 'ปิดรอบใหญ่ที่ #'.$bigLap;
			$this->datas['lap'] = 1;
			$this->datas['bigLap'] = $bigLap + 1;
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['count' => 0, 'wage' => 0, 'deposite' => 0])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
			$allUserss = $con->startQuery()
				->select(['userId', 'content'])
				->from($this->database)
				->where('id>1')
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			foreach ((array) $allUserss as $allUsers)
			{
				$contents = json_decode($allUsers['content'], 1);
				if (!empty($contents['maxWage'])) {
					unset($contents['maxWage']);
					$content = json_encode($contents);
					$con->startQuery()
						->update(['content' => $content])
						->set($this->database)
						->where(['userId' => $allUsers['userId']])
						->build()
						->query()
						->close();
				}
			}
		} else {
			$replyText = 'โปรดเช็คสถานะ';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function closeGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultClose($con, $this->database, $this->datas['game'], $resetTime);
			$replyText = 'ปิดบ้านแล้ว';
			$this->rebuildReport($con, $this->database);
			$this->datas['status'] = 'down';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'rollBack' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['tag' => 0, 'count' => 0, 'wage' => 0, 'deposite' => 0, 'rollBack' => null])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
			$allUserss = $con->startQuery()
				->select(['userId', 'content'])
				->from($this->database)
				->where('id>1')
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			foreach ((array) $allUserss as $allUsers)
			{
				$contents = json_decode($allUsers['content'], 1);
				if (!empty($contents['maxWage'])) {
					unset($contents['maxWage']);
					$content = json_encode($contents);
					$con->startQuery()
						->update(['content' => $content])
						->set($this->database)
						->where(['userId' => $allUsers['userId']])
						->build()
						->query()
						->close();
				}
			}
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'ยังไม่ปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			$replyText = 'ยังไม่สรุป';
		} else {
			$replyText = 'ปิดบ้านแล้ว';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function saveResult(DatabaseInterface $con, $text)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		$text = substr($text, 1);
		$explodeTexts = explode(',', $text);
		$sizeOfPlayer = sizeof($explodeTexts)-1;
		if ($sizeOfPlayer > $this->datas['player']) {
			$replyText = 'สรุปเกิน มี '.$this->datas['player'].' ขา';
			$error = 1;
		} else if ($sizeOfPlayer < $this->datas['player']) {
			$replyText = 'สรุปขาด มี '.$this->datas['player'].' ขา';
			$error = 1;
		} else if ($this->datas['status'] != 'check') {
			$replyText = 'กรุณาปิดรอบก่อน';
		} else {
			if (stripos($explodeTexts[0], '.')) {
				$hostValue = substr($explodeTexts[0], 1);
			} else {
				$hostValue = $explodeTexts[0][-1];
			}
			$results['host']['value'] = $hostValue;
			$deng = $explodeTexts[0][0];
			if ($deng == 1) {
				$results['host']['multi'] = 1;
				$replyText = "ขาเจ้า {$hostValue}แต้มไม่เด้ง\r\n";
			} else if ($deng <= $this->datas['deng']) {
				$results['host']['multi'] = $deng;
				$replyText = "ขาเจ้า {$hostValue}แต้ม{$deng}เด้ง\r\n";
			} else {
				$replyText = "ขาเจ้า สรุปผิด\r\n";
				$error = 1;
			}
			for ($player=1; $player<=$this->datas['player']; $player++) {
				if (stripos($explodeTexts[$player], '.')) {
					$playerValue = substr($explodeTexts[$player], 1);
				} else {
					$playerValue = $explodeTexts[$player][-1];
				}
				$results[$player]['value'] = $playerValue;
				$deng = $explodeTexts[$player][0];
				if ($deng == 1) {
					$results[$player]['multi'] = 1;
					$replyText .= "\r\nขา{$player} {$playerValue}แต้มไม่เด้ง";
				} else if ($deng <= $this->datas['deng']) {
					$results[$player]['multi'] = $deng;
					$replyText .= "\r\nขา{$player} {$playerValue}แต้ม{$deng}เด้ง";
				} else {
					$replyText .= "\r\nขา{$player} สรุปผิด";
					$error = 1;
				}
			}
			$replyText .= "\r\n\r\nยันยืนผลสรุป  yes หรือ @ok";
		}
		if ($error != 1) {
			$this->results = $results;
			$result = json_encode($this->results);
			$con->startQuery()
				->update(['result' => $result])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function submit(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if (empty($this->results) && $this->datas['status'] == 'check') {
			return 'สรุปผิด';
		} else if ($this->datas['status'] == 'check') {
			$con->startQuery()
				->update(['rollBack' => null])
				->set($this->database)
				->build()
				->query()
				->close();
			$recieveDatabases = $con->startQuery()
				->select(['id', 'content'])
				->from($this->database)
				->where('content IS NOT NULL')
				->and('content != ""')
				->and('content != "{}"')
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			foreach ((array) $recieveDatabases as $rows)
			{
				$con->startQuery()
					->update(['rollBack' => $rows['content']])
					->set($this->database)
					->where(['id' => $rows['id']])
					->build()
					->query()
					->close();
			}
			$con->startQuery()
				->update(['rollBack' => null])
				->set('money', 'money')
				->build()
				->query()
				->close();
			$replyText = $this->result($con, $this->database);
			$this->datas['lap']++;
			$this->datas['status'] = 'close';
			unset($this->results);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'result' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function setup(DatabaseInterface $con, $admins, $force = false)
	{
		if (!in_array($this->userId, $admins)) {
			return;
		}
		$database = $con->getDatabase();

		$existedDatabases = $con->startQuery()
			->select('SCHEMA_NAME')
			->from('SCHEMATA', 'information_schema')
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array($database, $existedDatabases)) {
			$con->startQuery()
				->create()
				->database($database)
				->build()
				->query()
				->close();
		}

		if (!in_array('money', $existedDatabases)) {
			$con->startQuery()
				->create()
				->database('money')
				->build()
				->query()
				->close();
		}

		$allTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => $database])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		$moneyTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => 'money'])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array('money', $moneyTables)) {
			$this->rebuildMoney($con);
			$install .= ' money';
		}
		if (!in_array('reportMoney', $moneyTables)) {
			$this->rebuildReportMoney($con);
			$install .= ' reportMoney';
		}
		if (!in_array('login', $allTables)) {
			$this->rebuildLogin($con);
			$install .= ' login';
		}
		if ($force === true || !in_array($this->database, $allTables)) {
			$this->rebuildPoke($con);
			$install .= ' poke';
		}
		if ($force === true || !in_array('reportPoke', $allTables)) {
			$this->rebuildReport($con, $this->database);
			$install .= ' reportPoke';
		}
		if ($force === true || !in_array('reportAllPoke', $allTables)) {
			$this->rebuildAllReport($con, $this->database);
			$install .= ' reportAllPoke';
		}

		if (!empty($install)) {
			$replyText = "ติดตั้ง$install สำเร็จ";
		} else {
			$replyText = "มีฐานข้อมูลอยู่แล้ว";
		}
		return $replyText;
	}

	protected function spacialResult(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		$resultRows = $con->startQuery()
			->select(['name', 'userId', 'content'])
			->from($this->database)
			->where('id > 1')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$whereText = $con->select(['userId'])
			->from($this->database)
			->where('id > 1')
			->build()
			->getSql();
		$allPlayerRows = $con->startQuery()
			->select(['id', 'userId', 'net', 'count', 'wage', 'bonus'])
			->from('money', 'money')
			->where("userId in ($whereText)")
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		foreach ((array) $allPlayerRows as $rows)
		{
			$idRows[$rows['userId']] = $rows['id'];
			$moneyRows[$rows['userId']] = $rows['net'];
			$countRows[$rows['userId']] = (int) $rows['count']+1;
			$wageRows[$rows['userId']] = (int) $rows['wage'];
			$bonusRows[$rows['userId']] = (int) $rows['bonus'];
		}
		$hostValue = $this->results['host']['value'];
		$replyTexts = $this->flexMessage([
			'altText' => "สรุปรอบย่อยที่ #".$this->datas['lap'],
			'header' => "สรุปรอบย่อยที่ #".$this->datas['lap'],
		], 'xs');
		$normalReply = "สรุปรอบย่อยที่ #".$this->datas['lap'];
		foreach ((array) $resultRows as $rows)
		{
			unset($allWageNet, $money, $maxWage);
			$contents = json_decode($rows['content'], 1);
			$contentMulti = $contents['multi'];
			$maxWage = $contents['maxWage'];
			unset($contents['multi'], $contents['maxWage']);
			$userNet = $moneyRows[$rows['userId']];
			$userBonus = $bonusRows[$rows['userId']];
			$rollBacks = [
				'net'   => $userNet,
				'count' => $countRows[$rows['userId']]-1,
				'wage'  => $wageRows[$rows['userId']],
				'bonus' => $userBonus,
			];
			foreach ((array) $contents as $wage)
			{
				$maxWage = $maxWage < $wage ? $wage : $maxWage;
				$allWageNet += (int) $wage;
			}
			$allWageNet *= $contentMulti == 'yes' ? $this->datas['deng'] : 1;
			if ($allWageNet == 0 || $allWageNet > $userNet+$userBonus) {
				continue;
			}
			foreach ((array) $contents as $position => $content)
			{
				$playerValue = $this->results[$position]['value'];
				if ($contentMulti == 'yes') {
					$hostMulti = $this->results['host']['multi'];
					$playermulti = $this->results[$position]['multi'];
				} else {
					$hostMulti = 1;
					$playermulti = 1;
				}
				if ($playerValue > $hostValue) {
					$money += $content * $playermulti;
					$outcome += $content * $playermulti;
				} else if ($playerValue < $hostValue) {
					$money -= $content * $hostMulti;
					$income += $content * $hostMulti;
				}
				$wageRows[$rows['userId']] += $content;
			}
			if ($userBonus > 0 && $money < 0) {
				$userBonus += $money;
			} else {
				$userNet += $money;
			}
			if ($userBonus <= 0) {
				$userNet += $userBonus;
				$userBonus = 0;
			}
			if ($money > 0) {
				$eachReply = "+$money";
			} else if ($money == 0) {
				$eachReply = "+0";
			} else {
				$eachReply = "$money";
			}
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						$idRows[$rows['userId']].")".json_decode($rows['name'], 1),
						$eachReply,
						$userNet,
						"💸".(int) $userBonus,
					],
				],
			], 'xs');
			$normalReply .= "\r\n".$idRows[$rows['userId']].")".json_decode($rows['name'], 1).$eachReply."=".$userNet."💸".(int) $userBonus;
			$replyContent = json_encode(['multi' => $contentMulti, 'maxWage' => (int)$maxWage]);
			$rollBack = json_encode($rollBacks);
			$con->startQuery()
				->update([
					'net'      => $userNet,
					'count'    => $countRows[$rows['userId']],
					'wage'     => $wageRows[$rows['userId']],
					'bonus'    => $userBonus,
					'rollBack' => $rollBack,
				])
				->set('money', 'money')
				->where(['userId' => $rows['userId']])
				->build()
				->query()
				->close();
			$con->startQuery()
				->update(['content' => $replyContent])
				->set($this->database)
				->where(['userId' => $rows['userId']])
				->build()
				->query()
				->close();
		}
		$reports = [
			'game'    => $this->datas['game'],
			'bigLap'  => $this->datas['bigLap'],
			'lap'     => $this->datas['lap'],
			'income'  => $income,
			'outcome' => $outcome,
		];
		$this->report($con, $this->database, $reports);
		$replyTexts['normalReply'] = $normalReply;

		return $replyTexts;
	}
}