<?php
namespace Koob\TigerDragonBundle\Model;

use Koob\InterfaceBundle\TigerDragonInterface;
use Koob\InterfaceBundle\DatabaseInterface;
use Koob\GameBundle\Model\GameModel;

class FunctionModel extends GameModel implements TigerDragonInterface
{
	protected $database = 'tigerDragon';

	protected $attrs = ['min', 'max', 'reply', 'color', 'depo', 'allDepo', 'rule', 'deli', 'allDeli', 'recc', 'sc', 'cp', '$', 'jackpot', 'report', 'showsc'];

	public function rebuildTigerDragon(DatabaseInterface $con)
	{
		$con->startQuery()
			->drop()
			->table($this->database)
			->build()
			->query()
			->close();

		$con->startQuery()
			->create([
				'name'     => 'VARCHAR(255)',
				'userId'   => 'VARCHAR(255) UNIQUE',
				'admin'    => 'VARCHAR(255)',
				'content'  => 'TEXT',
				'result'   => 'TEXT',
				'rollBack' => 'TEXT',
			])
			->table($this->database)
			->build()
			->query()
			->close();

		$datas = [
			'status' => 'down',
			'game'   => 0,
			'bigLap' => 1,
			'lap'    => 1,
			'min'    => 20,
			'max'    => 200,
			'reply'  => 'yes',
		];
		$data = json_encode($datas);
		$con->startQuery()
			->insert([
				'name' => '"index"',
				'userId' => 'index',
				'content' => $data,
			])
			->into($this->database)
			->build()
			->query()
			->close();
	}

	public function wage(DatabaseInterface $con, $text)
	{
		$hasDeli = stripos($text, $this->datas['deli']);
		if ($this->datas['status'] == 'down') {
			return;
		} else if ($this->datas['status'] == 'close') {
			return 'ยังไม่เปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			return 'แทงไม่ทัน';
		} else if (!$hasDeli) {
			return "รูปแบบการแทง xx".$this->datas['deli']."xxx";
		}
		$explodeTexts = explode($this->datas['allDeli'], $text);
		foreach ((array) $explodeTexts as $wageList)
		{
			[$type, $wageNet] = explode($this->datas['deli'], $wageList);
			if ((int) $this->datas['rule'] == 0) {
				if (strlen($type) == 1 && !in_array($type, ['t', 'd', '1', '2', 'b', 'r'])) {
					$errors[] = "การแทง $wageList ขาไม่ถูกต้อง❗";
					continue;
				}
				if (in_array($type, ['1', '2', 'b', 'r']) && $this->datas['color'] <= 0) {
					$errors[] = "การแทง $wageList แทงสีคู่คี่ไม่ได้แล้ว❗";
					continue;
				}
				if (strlen($type) != 1 && !in_array($type, ['td'])) {
					$errors[] = "การแทง $wageList ขาไม่ถูกต้อง❗";
					continue;
				}
				if (!is_numeric($wageNet)) {
					$errors[] = "การแทง $wageList ยอดเงินผิด";
					continue;
				}
				// if (strlen($type) != 1 && !in_array($type, ['td', 'tb', 'tr', 't1', 't2', 'db', 'dr', 'd1', 'd2'])) {
				// 	$errors[] = "การแทง $wageList ขาไม่ถูกต้อง❗";
				// 	continue;
				// }
			} else if ((int) $this->datas['rule'] == 1) {
				if ($type == 't') {
					$type = 1;
				}
				if ($type == 'd') {
					$type = 2;
				}
				if ($type == 'p') {
					$type = 3;
				}
				if ($type == 'e') {
					$type = 4;
				}
				if (strlen($type) == 1 && !in_array($type, [1, 2, 3, 4])) {
					$errors[] = "การแทง $wageList ขาไม่ถูกต้อง❗";
					continue;
				}
				if (strlen($type) != 1) {
					$errors[] = "การแทง $wageList ขาไม่ถูกต้อง❗";
					continue;
				}
				if (!is_numeric($wageNet)) {
					$errors[] = "การแทง $wageList ยอดเงินผิด";
					continue;
				}
			}
			if (!($this->datas['min'] <= $wageNet && $wageNet <= $this->datas['max'])) {
				$errors[] = "การแทง $wageList ไม่ถูกต้อง วงเงินการแทง ".$this->datas['min'].'-'.$this->datas['max'].' บาท❗';
				continue;
			}
			if (!empty($this->datas['depo']) && !empty($this->bonus) && $this->datas['depo'] > 0) {
				$max = $this->deposite*$this->datas['depo']/100;
				$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
				if ($wageNet < $maximum) {
					$errors[] = "แทง ".$this->datas['depo']."% ของยอดฝาก($max ขึ้นไป)";
					continue;
				}
			}
			if (!empty($this->datas['cp']) && $this->play != 'playing' && $this->checkPeople($con, $this->database, true) >= $this->datas['cp']) {
				$errors[] = "จำนวนคนครบ ".$this->datas['cp']." คนแล้ว";
				continue;
			}
			$this->contents[$type] = $wageNet;
		}
		foreach ((array) $this->contents as $wageNet)
		{
			$allWageNet += $wageNet;
		}
		if (!empty($this->datas['allDepo']) && !empty($this->bonus) && $this->datas['allDepo'] > 0)  {
			$max = $this->deposite*$this->datas['allDepo']/100;
			$maximum = $max > $this->datas['max'] ? $this->datas['max'] : $max;
			if ($allWageNet < $maximum) {
				$errors[] = "แทงทั้งหมด ".$this->datas['allDepo']."% ของยอดฝาก($max ขึ้นไป)";
			}
		}
		if ($allWageNet <= $this->net+$this->bonus) {
			$errorText = implode(" ", (array) $errors);
		} else {
			$errorText = "เครดิตไม่เพียงพอ❗";
		}
		$clearErrorText = str_replace([' ', "\r\n"], '', $errorText);
		if (empty($clearErrorText)) {
			$content = json_encode($this->contents);
			$con->startQuery()
				->update(['content' => $content, 'result' => 'playing'])
				->set($this->database)
				->where(['userId' => $this->userId])
				->build()
				->query()
				->close();
		}
		$replyTexts = $this->check();
		if ($this->datas['reply'] == 'yes') {
			if (!empty($errorText)) {
				$replyTexts .= "\r\n\r\n$errorText";
			}
		} else if ($this->datas['reply'] == 'flex') {
			if (!empty($clearErrorText)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							$errorText,
							'separator #000000',
						],
					],
				], 'xs');
			}
		} else {
			unset($replyTexts);
		}
		return $replyTexts;
	}

	public function unWage(DatabaseInterface $con, $text)
	{
		if ($this->datas['status'] != 'open') {
			return $this->datas['status'] == 'down' ? '' : "$this->name ปิดรอบแล้ว❗";
		}
		foreach ((array) $this->contents as $type => $wage)
		{
			unset($this->contents[$type]);
			$cancel .= " $type";
		}
		$content = json_encode($this->contents);
		$con->startQuery()
			->update(['content' => $content, 'result' => null])
			->set($this->database)
			->where(['userId' => $this->userId])
			->build()
			->query()
			->close();
		if (!empty($cancel)) {
			if ($this->datas['reply'] == 'flex') {
				$replyTexts = $this->flexMessage([
					'old' => $this->check(),
					'body' => [
						[
							"ยกเลิกขา$cancel",
							"separator #000000",
						],
					],
				], 'xs');
			} else if ($this->datas['reply'] == 'yes') {
				$replyTexts = $this->check()."\r\n\r\nยกเลิกขา$cancel";
			}
		}
		return $replyTexts;
	}

	public function unWageAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		$con->startQuery()
			->update(['content' => '{}', 'result' => null])
			->set($this->database)
			->where('id>1')
			->build()
			->query()
			->close();
		return "ยกเลิกการแทงทั้งหมด";
	}

	public function check()
	{
		$replyTexts = $this->flexMessage([
			'altText' => "สรุปการแทงของ $this->name",
			'footer' =>	"{$this->net}💸{$this->bonus}",
			'body' => [
				[
					$this->name,
				],
			],
		], 'xs');
		$isEmpty = 1;
		$text = "$this->name แทง";
		foreach ((array) $this->contents as $type => $wageNet)
		{
			$isEmpty = 0;
			if ((int) $this->datas['rule'] == 1) {
				if ($type == 1) $type = "เสือ";
				if ($type == 2) $type = "มังกร";
				if ($type == 3) $type = "เสมอ";
				if ($type == 4) $type = "เสมอคู่เหมือน";
			}
			$text .= " $type=$wageNet";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						$type,
						$wageNet,
					],
				],
			], 'xs');
		}
		if ($isEmpty) {
			$text .= "ไม่ได้แทง";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						'ไม่ได้แทง',
					],
				],
			], 'xs');
		}
		$text .= "\r\n{$this->net}💸{$this->bonus}";
		$replyTexts = $this->flexMessage([
			'old' => $replyTexts,
			'body' => [
				[
					'separator #000000',
				],
			],
		]);
		if ($this->datas['reply'] == 'yes') {
			$replyTexts = $text;
		}
		return $replyTexts;
	}

	public function checkAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}

		$resultRows = $con->startQuery()
			->select(['x.name, x.content, m.userId', 'm.net', 'm.bonus'])
			->from($this->database.' x, money.money m')
			->where('x.userId = m.userId')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$replyTexts = $this->flexMessage([
			'altText' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
			'header' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
		]);
		foreach ((array) $resultRows as $rows)
		{
			$contents = json_decode($rows['content'], 1);
			$name = json_decode($rows['name'], 1);
			foreach ((array) $contents as $type => $wageNet)
			{
				$reply .= " $type=$wageNet";
			}
			if ($rows['bonus'] > 0) {
				$bonusText = "💸".$rows['bonus']."=".($rows['bonus']+$rows['net']);
			}
			if (!empty($reply)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							"$name แทง$reply 💰".$rows['net']."{$bonusText}{$suffix}",
						],
					],
				]);
			}
			unset($reply, $bonusText);
		}
		$noPlayerReplys = [
			"type" => "text",
			"text" => "ยังไม่มีผู้เล่นแทง",
		];
		if (empty($replyTexts['messages'][0]['contents']['contents'][0]['body'])) {
			$replyTexts = [
				'messages' => [$noPlayerReplys],
				'raw' => true
			];
		}
		if ($this->datas['report'] != 'no') {
			$replyTexts['report'] = $this->allWage($resultRows);
		}
		return $replyTexts;
	}

	public function openGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'down') {
			$sendText = $this->resultOpen($con, $this->database, $this->datas['game'], $resetTime);
			$game = $this->datas['game'] + 1;
			$replyText = "เริ่มเกมที่ #".$game;
			$this->datas['status'] = 'close';
			$this->datas['game'] = $game;
			$this->datas['bigLap'] = 1;
			$this->datas['lap'] = 1;
			$this->datas['startTime'] = date('H.i');
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else {
			$replyText = "เปิดบ้านแล้ว";
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function openRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			// $replyText[] = $locals['openMessage'];
			// $replyText[] = "เปิดรอบย่อยที่ #".$this->datas['lap'];
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			if ($this->datas['sc'] != 'no' && !empty($this->scores)) {
				$replyText["messages"][] = $this->showScore($con, $this->database)['messages'][0];
			}
			$this->datas['status'] = 'open';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'เปิดรอบแล้ว';
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'กรุณาสรุปผลให้เรียบร้อยก่อน';
		}
		return $replyText;
	}

	public function closeRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'open') {
			// $replyText[] = $locals['closeMessage'];
			// $replyText[] = 'ปิดรอบย่อยที่ #' . $this->datas['lap'];
			$checkAll = $this->checkAll($con);
			$replyText['report'] = $checkAll['report'];
			unset($checkAll['report']);
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				$checkAll["messages"][0],
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			if ($this->datas['report'] == 'no') {
				unset($replyText['report']);
			}
			$this->datas['status'] = 'check';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'rollBack' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'ปิดรอบแล้ว';
		}
		return $replyText;
	}

	public function restRound(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultRest($con, $this->database, $this->datas['game'], $this->datas['bigLap'], $resetTime, $this->datas['startTime']);
			$bigLap = $this->datas['bigLap'];
			$replyText = 'ปิดรอบใหญ่ที่ #'.$bigLap;
			$this->datas['lap'] = 1;
			$this->datas['bigLap'] = $bigLap + 1;
			unset($this->datas['isJackpotOut']);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'admin' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['count' => 0, 'wage' => 0, 'deposite' => 0])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else {
			$replyText = 'โปรดเช็คสถานะ';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function closeGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultClose($con, $this->database, $this->datas['game'], $resetTime);
			$replyText = 'ปิดบ้านแล้ว';
			$this->rebuildReport($con, $this->database);
			$this->datas['status'] = 'down';
			unset($this->datas['isJackpotOut']);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'admin' => null, 'rollBack' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['tag' => 0, 'count' => 0, 'wage' => 0, 'deposite' => 0, 'rollBack' => null])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'ยังไม่ปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			$replyText = 'ยังไม่สรุป';
		} else {
			$replyText = 'ปิดบ้านแล้ว';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function saveResult(DatabaseInterface $con, $text)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] == 'down' || $this->datas['status'] == 'close') {
			return;
		}
		if ($this->datas['status'] != 'check') {
			return 'กรุณาปิดรอบก่อน';
		}
		$text = substr($text, 1);
		[$tiger, $dragon] = explode(',', $text);
		$tigerC = $tiger[0];
		$tiger = substr($tiger, 1);
		$dragonC = $dragon[0];
		$dragon = substr($dragon, 1);
		if (!is_numeric($tiger) || !is_numeric($dragon)) {
			$error = 1;
		}
		if ((int) $this->datas['rule'] == 0) {
			if (!in_array($tigerC, ['b', 'r']) || !in_array($dragonC, ['b', 'r'])) {
				$error = 1;
			}
			$tempWordTiger = $tigerC == 'b' ? "ดำ" : "แดง";
			$tempWordDragon = $dragonC == 'b' ? "ดำ" : "แดง";
		} else if ((int) $this->datas['rule'] == 1) {
			if (!in_array($tigerC, [1, 2, 3, 4]) || !in_array($dragonC, [1, 2, 3, 4])) {
				$error = 1;
			}
			if ($tigerC == 1) {
				$tempWordTiger = "♣";
			} else if ($tigerC == 2) {
				$tempWordTiger = "♦";
			} else if ($tigerC == 3) {
				$tempWordTiger = "♥";
			} else {
				$tempWordTiger = "♠";
			}
			if ($dragonC == 1) {
				$tempWordDragon = "♣";
			} else if ($dragonC == 2) {
				$tempWordDragon = "♦";
			} else if ($dragonC == 3) {
				$tempWordDragon = "♥";
			} else {
				$tempWordDragon = "♠";
			}
		}
		$displays = [0,1,2,3,4,5,6,7,8,9,10,'j','q','k'];
		$this->results['tiger']['color'] = $tigerC;
		$replyText .= "เสือ {$displays[$tiger]}{$tempWordTiger}, ";
		$this->results['dragon']['color'] = $dragonC;
		$replyText .= "มังกร {$displays[$dragon]}{$tempWordDragon}";
		if (0 > $tiger || $tiger > 13 || 0 > $dragon || $dragon > 13) {
			$error = 1;
		} else {
			$this->results['tiger']['value'] = $tiger;
			$this->results['dragon']['value'] = $dragon;
		}
		$replyText = empty($error) ? $replyText : "สรุปผิด กรุณาสรุปใหม่";
		$replyText .= "\r\n\r\nยันยืนผลสรุป yes หรือ @ok";
		$result = empty($error) ? json_encode($this->results) : null;
		$con->startQuery()
			->update(['result' => $result])
			->set($this->database)
			->where(['id' => 1])
			->build()
			->query()
			->close();
		return $replyText;
	}

	public function submit(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] != 'check') {
			return;
		}
		if (empty($this->results)) {
			return 'สรุปผิด';
		} else {
			$this->datas['color'] = $this->datas['color'] == -1 ? $this->datas['color'] : $this->datas['color']-1;
			$con->startQuery()
				->update(['rollBack' => null])
				->set($this->database)
				->build()
				->query()
				->close();
			$recieveDatabases = $con->startQuery()
				->select(['id', 'content'])
				->from($this->database)
				->where('content IS NOT NULL')
				->and('content != ""')
				->and('content != "{}"')
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			foreach ((array) $recieveDatabases as $rows)
			{
				$con->startQuery()
					->update(['rollBack' => $rows['content']])
					->set($this->database)
					->where(['id' => $rows['id']])
					->build()
					->query()
					->close();
			}
			$con->startQuery()
				->update(['rollBack' => null])
				->set('money', 'money')
				->build()
				->query()
				->close();
			$replyText = $this->result($con, $this->database);
			if ($this->datas['color'] == 0) {
				$replyText['messages'][] = [
					"type" => "text",
					"text" => "หมดรอบการแทงสีคู่คี่",
				];
			}
			$this->datas['lap']++;
			$this->datas['status'] = 'close';
			unset($this->results);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'result' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function setup(DatabaseInterface $con, $admins, $force = false)
	{
		if (!in_array($this->userId, $admins)) {
			return;
		}
		$database = $con->getDatabase();

		$existedDatabases = $result = $con->startQuery()
			->select('SCHEMA_NAME')
			->from('SCHEMATA', 'information_schema')
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array($database, $existedDatabases)) {
			$con->startQuery()
				->create()
				->database($database)
				->build()
				->query()
				->close();
		}

		if (!in_array('money', $existedDatabases)) {
			$con->startQuery()
				->create()
				->database('money')
				->build()
				->query()
				->close();
		}

		$allTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => $database])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		$moneyTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => 'money'])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array('money', $moneyTables)) {
			$this->rebuildMoney($con);
			$install .= ' money';
		}
		if (!in_array('reportMoney', $moneyTables)) {
			$this->rebuildReportMoney($con);
			$install .= ' reportMoney';
		}
		if (!in_array('login', $allTables)) {
			$this->rebuildLogin($con);
			$install .= ' login';
		}
		if ($force === true || !in_array($this->database, $allTables)) {
			$this->rebuildTigerDragon($con);
			$install .= ' tigerDragon';
		}
		if ($force === true || !in_array('reportTigerDragon', $allTables)) {
			$this->rebuildReport($con, $this->database);
			$install .= ' reportTigerDragon';
		}
		if ($force === true || !in_array('reportAllTigerDragon', $allTables)) {
			$this->rebuildAllReport($con, $this->database);
			$install .= ' reportAllTigerDragon';
		}

		if (!empty($install)) {
			$replyText = "ติดตั้ง$install สำเร็จ";
		} else {
			$replyText = "มีฐานข้อมูลอยู่แล้ว";
		}
		return $replyText;
	}

	protected function saveScore(DatabaseInterface $con)
	{
		$tiger = $this->results['tiger']['value'];
		$dragon = $this->results['dragon']['value'];
		$tigerC = $this->results['tiger']['color'];
		$dragonC = $this->results['dragon']['color'];
		if ($tiger > $dragon) {
			$nextScore = 1;
		} else if ($tiger < $dragon) {
			$nextScore = 2;
		} else if ($tigerC > $dragonC) {
			$nextScore = 3;
		} else if ($tigerC < $dragonC) {
			$nextScore = 4;
		} else {
			$nextScore = 5;
		}
		$this->scores[] = $nextScore;
		$score = json_encode($this->scores);
		$con->startQuery()
			->update(['admin' => $score])
			->set($this->database)
			->where(['id' => 1])
			->build()
			->query()
			->close();
	}

	protected function rule()
	{
		if ((int) $this->datas['rule'] == 0) {
			$rewards = [
				't'  => 1,
				'd'  => 1,
				'td' => 8,
				'1'  => 4/3,
				'2'  => 1.5,
				'b'  => 10/9,
				'r'  => 10/9,
			];
			if ($this->results['tiger']['value'] % 2 == 1 || $this->results['dragon']['value'] % 2 == 1)
				$results[] = '1';
			if ($this->results['tiger']['value'] % 2 == 0 || $this->results['dragon']['value'] % 2 == 0)
				$results[] = '2';
			if ($this->results['tiger']['color'] == 'b' || $this->results['dragon']['color'] == 'b')
				$results[] = 'b';
			if ($this->results['tiger']['color'] == 'r' || $this->results['dragon']['color'] == 'r')
				$results[] = 'r';
			if ($this->results['tiger']['value'] > $this->results['dragon']['value']) {
				$results[] = 't';
				if ($this->results['tiger']['value'] % 2 == 1)
					$results[] = 't1';
				else
					$results[] = 't2';
				if ($this->results['tiger']['color'] == 'b')
					$results[] = 'tb';
				else
					$results[] = 'tr';
			} else if ($this->results['tiger']['value'] < $this->results['dragon']['value']) {
				$results[] = 'd';
				if ($this->results['dragon']['value'] % 2 == 1)
					$results[] = 'd1';
				else
					$results[] = 'd2';
				if ($this->results['dragon']['color'] == 'b')
					$results[] = 'db';
				else
					$results[] = 'dr';
			} else {
				$results[] = 'td';
				$reduces['t'] = -0.5;
				$reduces['d'] = -0.5;
			}
		} else if ((int) $this->datas['rule'] == 1) {
			$rewards = [
				'1' => 1,
				'2' => 1,
				'3' => 8,
				'4' => 30,
			];
			if ($this->results['tiger']['value'] > $this->results['dragon']['value']) {
				$results[] = '1';
			} else if ($this->results['tiger']['value'] < $this->results['dragon']['value']) {
				$results[] = '2';
			} else {
				$results[] = '3';
				if ($this->results['tiger']['color'] > $this->results['dragon']['color']) {
					$results[] = '1';
				} else if ($this->results['tiger']['color'] < $this->results['dragon']['color']) {
					$results[] = '2';
				} else {
					$results[] = '4';
				}
			}
		}
		$rewards['jackpot'] = 200;
		$rewards['jackpotOne'] = 5000;
		$results[] = 'jackpot';
		$results[] = 'jackpotOne';
		return [$rewards, $results, $reduces];
	}

	protected function beforeResultLoop($resultRows, $results)
	{
		if ($this->datas['jackpot'] != 'yes') {
			return $resultRows;
		}
		if ($this->results['tiger']['color'] == 4 && in_array($this->results['tiger']['value'], [11, 12, 13]) && in_array(1, $results)) {
			$winner = 1;
		} else if ($this->results['dragon']['color'] == 4 && in_array($this->results['dragon']['value'], [11, 12, 13]) && in_array(2, $results)) {
			$winner = 2;
		} else if ($this->results['tiger']['color'] == 4 && in_array(4, $results)) {
			$winner = 4;
		} else {
			return $resultRows;
		}
		foreach ((array) $resultRows as $n => $rows) {
			$contents = json_decode($rows['content'], 1);
			$allWageNet = 0;
			foreach ((array) $contents as $wage)
			{
				$allWageNet += $wage;
			}
			if (empty($allWageNet)) {
				continue;
			}
			if ($winner != 4 && $allWageNet >= 500 && in_array($winner, array_keys($contents))) {
				$contents['jackpot'] = 1;
				$resultRows[$n]['content'] = json_encode($contents);
			} else if (empty($this->datas['isJackpotOut']) && $allWageNet >= 3000 && $winner == 4) {
				if ($maxValue < $allWageNet) {
					unset($maxPlayers);
					$maxPlayers[] = $n;
					$maxValue = $allWageNet;
				} else if ($maxValue == $allWageNet) {
					$maxPlayers[] = $n;
				}
			}
		}
		if (!empty($maxPlayers)) {
			foreach ((array) $maxPlayers as $n)
			{
				$contents = json_decode($resultRows[$n]['content'], 1);
				$contents['jackpotOne'] = 1/count($maxPlayers);
				$resultRows[$n]['content'] = json_encode($contents);
			}
			$this->datas['isJackpotOut'] = 1;
		}
		return $resultRows;
	}

	protected function allWage($resultRows)
	{
		foreach ((array) $resultRows as $rows)
		{
			$contents = json_decode($rows['content'], 1);
			foreach ((array) $contents as $n => $wage)
			{
				$name = $n == 1 ? "เสือ" : ($n == 2 ? "มังกร" : ($n == 3 ? "เสมอ" : "เสมอคู่เหมือน"));
				$wages[$name] += $wage;
			}
		}
		$reportText = "สรุปการแทง";
		foreach ((array) $wages as $name => $wage)
		{
			$reportText .= "\r\n$name $wage";
		}
		if (empty($wages)) {
			unset($reportText);
		}
		return $reportText;
	}
}
