<?php
namespace Koob\TigerDragonBundle\Model;

use Koob\InterfaceBundle\LobbyInterface;
use Koob\DatabaseBundle\Controller\DatabaseController;
use Koob\LineApiBundle\Controller\LineApiController;
use Koob\TigerDragonBundle\Controller\FunctionController;
use Koob\LocalBundle\Controller\LocalController;

class LobbyModel implements LobbyInterface
{
	protected $con;

	protected $api;

	protected $func;

	protected $local;

	protected $database = 'tigerDragon';

	protected $groupId;

	function __construct()
	{
		$this->con = new DatabaseController;
		$this->api = new LineApiController;
		$this->func = new FunctionController;
		$this->local = new LocalController;

		$this->con->setHost($this->local->getParameter('host'))
			->setUser($this->local->getParameter('user'))
			->setPass($this->local->getParameter('pass'))
			->setDatabase($this->local->getParameter('database'));

		$this->api->setAccessToken($this->local->getParameter('accessToken'));
	}

	public function run($content)
	{
		if (!empty($content['events'])) {
			foreach ((array) $content['events'] as $events)
			{
				$admins           = $this->local->getParameter('admins');
				$allowGroups      = $this->local->getParameter('allowGroups');
				$ignoreUserIds    = $this->local->getParameter('ignoreUserIds');
				$forceRebuild     = $this->local->getParameter('forceRebuild');
				$reportRoom       = $this->local->getParameter('reportRoom');
				$openMessage      = $this->local->getParameter('gameMessages')['openMessage'];
				$closeMessage     = $this->local->getParameter('gameMessages')['closeMessage'];
				$openPic          = $this->local->getParameter('pictures')['open'];
				$closePic         = $this->local->getParameter('pictures')['close'];
				$autoReplyMessage = $this->local->getParameter('gameMessages')['autoReplyMessage'];
				$resetTime        = $this->local->getParameter('resetTime');
				$tags             = $this->local->getParameter('tags');
				$playRooms        = $this->local->getParameter('playRoom');
				$withdrawRoom     = $this->local->getParameter('withdrawRoom');

				$groupId = $events['source']['groupId'];
				$userId = $events['source']['userId'];
				$this->groupId = $groupId;

				if (in_array($groupId, $allowGroups) || in_array($userId, $allowGroups)) {
					$type = $events['message']['type'];
					$originalText = $events['message']['text'];
					$packageId = $events['message']['packageId'];
					$stickerId = $events['message']['stickerId'];
					$roomId = $events['source']['roomId'];

					$ids = [
						'userId'  => $userId,
						'roomId'  => $roomId,
						'groupId' => $groupId,
					];
					$userId = $this->func->getApi($this->api, $ids);

					$id = $this->func->getData($this->con, $this->database, $userId);

					//clear zero width space
					$originalText = json_decode(str_replace('\u200b', '', json_encode($originalText)), 1);
					$text = str_replace([' ', /*"\r\n"*/], '', $originalText);
					$text = strtolower($text);
					$explodeTexts = explode(' ', $originalText);

					if (empty($id) && !empty($userId)) {
						$id = $this->func->insert($this->con, $this->database);
					}

					if (in_array($text, ['ox'])) {
						$replyText = "id:$id\r\nuserId:$userId";
					}

					if ($originalText == 'ALL') {
						$replyText = "groupId:$groupId\r\nRoomId:$roomId\r\nuserId:$userId";
					}

					if (in_array($text[0], ['x'])) {
						$replyText = $this->func->unWage($this->con, $text);
					}

					if (in_array($originalText, ['cancel all'])) {
						$replyText = $this->func->unWageAll($this->con);
					}

					if (in_array($text, ['check', 'ch', 'c', 'เช็ค'])) {
						$replyText = $this->func->check(true);
					}

					if ($type == 'image' || substr($text, 0, 3) == '@id') {
						$replyText = $this->func->checkMoney($this->con, $this->database, $ignoreUserIds);
					}

					if (([$type, $packageId, $stickerId] == ['sticker', '2000003', '48844']) || in_array($originalText, ['openGame', '(open door)'])) {
						[$replyText, $sendText] = $this->func->openGame($this->con, $resetTime);
						$this->send($reportRoom, $sendText);
					}

					if (([$type, $packageId, $stickerId] == ['sticker', '2000004', '48982']) || in_array($originalText, ['openRound', '(on arrow)'])) {
						$replyText = $this->func->openRound($this->con, ['openMessage' => $openMessage, 'pic' => $openPic]);
					}

					if (([$type, $packageId, $stickerId] == ['sticker', '2000004', '48988']) || in_array($originalText, ['closeRound', '(smartphone turned off)'])) {
						$replyText = $this->func->closeRound($this->con, ['closeMessage' => $closeMessage, 'pic' => $closePic]);
						if (is_array($replyText)) {
							$reportText = $replyText['report'];
							$this->send($reportRoom, $reportText);
							unset($replyText['report']);
						}
					}

					if (([$type, $packageId, $stickerId] == ['sticker', '2000004', '48963']) || in_array($originalText, ['restRound', '(sync)'])) {
						[$replyText, $sendText] = $this->func->restRound($this->con, $resetTime);
						$this->send($reportRoom, $sendText);
					}

					if (($type == 'sticker' && $packageId == '2000003' && $stickerId == '48843') || in_array($originalText, ['closeGame', '(closed door)'])) {
						[$replyText, $sendText] = $this->func->closeGame($this->con, $resetTime);
						$this->send($reportRoom, $sendText);
					}

					if (in_array($text, ['checkall', 'ca', 'a', 'สรุป'])) {
						$replyText = $this->func->checkAll($this->con);
						if (is_array($replyText)) {
							$reportText = $replyText['report'];
							$this->send($reportRoom, $reportText);
							unset($replyText['report']);
						}
					}

					if (in_array($text[0], ['s'])) {
						if (in_array($text, ['sc'])) {
							$replyText = $this->func->showScore($this->con, $this->database);
						} else if ($text == 'status') {
							$replyText = $this->func->status($this->con, $admins);
						} else if (in_array($text, ['setup'])) {
							$replyText = $this->func->setup($this->con, $admins, $forceRebuild);
						} else if (in_array($explodeTexts[0], ['setadmin'])) {
							$replyText = $this->func->setAdmin($this->con, $this->database, $admins, $explodeTexts[1], $explodeTexts[2]);
						} else {
							$replyText = $this->func->saveResult($this->con, $text);
						}
					}

					if (in_array($text, ['r'])) {
						$replyText = $this->func->saveRollBack($this->con, $this->database);
					}

					if (in_array($text, ['@ok', 'yes', 'y', '@'])) {
						$replyText = $this->func->submit($this->con);
						$normalReply = $replyText['normalReply'];
						unset($replyText['normalReply']);
						foreach ((array) $playRooms as $playRoom) {
							if ($playRoom == $groupId) {
								continue;
							}
							$this->send($playRoom, $normalReply);
						}
					}
					
					if (in_array($explodeTexts[0], ['@wd'])) {
						$replyText = $this->func->withdraw($this->con, $this->database, $originalText);
					}

					if (in_array(substr($originalText, 0, 3), ['#WD'])) {
						['reply' => $replyText, 'report' => $sendText] = $this->func->confirmWithdraw($this->con, $this->database, $originalText);
						$this->send($withdrawRoom, $sendText);
					}

					if (in_array($text, ['delmoney'])) {
						$replyText = $this->func->deleteAllMoney($this->con, $this->database);
					}

					if (in_array(substr($originalText, 0, 4), ['#DAM'])) {
						$replyText = $this->func->deleteAllMoney($this->con, $this->database, $originalText);
					}

					if (in_array($explodeTexts[0], ['use'])) {
						$replyText = $this->func->changeAttr($this->con, $explodeTexts[1], $explodeTexts[2]);
					}

					if ($text[0] == '$'){
						$replyText = $this->func->fixMoney($this->con, $this->database, $text);
					}

					if (in_array($explodeTexts[0], ['checkid'])) {
						$replyText = $this->func->checkId($this->con, $this->database, $admins, $explodeTexts);
					}

					if (in_array($text, ['cc', 'เช็ครอบ'])) {
						$replyText = $this->func->checkCount($this->con, $this->database, $tags);
					}

					if (in_array($text, ['cm', 'เช็คยอด'])) {
						$replyText = $this->func->checkAllMoney($this->con, $this->database, $groupId == $reportRoom);
					}

					if (in_array($text, ['cp', 'เช็คคน'])) {
						$replyText = $this->func->checkPeople($this->con, $this->database);
					}

					if ($explodeTexts[0] == 'test') {
						$replyText = $this->func->test($originalText);
					}

					if (empty($replyText) && $type == 'text') {
						$replyText = $this->func->wage($this->con, $text);
					}
				} else {
					$replyText = $autoReplyMessage."\r\ngroupId : $groupId";
				}
				return $replyText;
			}
		}
	}

	public function send($destination, $replyTexts)
	{
		$this->api->setMethod('push')
			->startApi()
			->setDestination($destination)
			->setIgnore([$this->groupId])
			->setMessage($replyTexts)
			->send()
			->close();
	}
}