<?php
namespace Koob\ViewBundle\Model;

use Koob\InterfaceBundle\ViewInterface;
use Koob\InterfaceBundle\DatabaseInterface;
use Koob\LocalBundle\Controller\LocalDatabaseController;

class FunctionModel implements ViewInterface
{
	protected $search;

	protected $localDatabase;

	function __construct()
	{
		$this->localDatabase = new LocalDatabaseController;
	}

	public function setupTable(DatabaseInterface $con, $resetTime, $game)
	{
		$resultRows = $con->startQuery()
			->select(['id', 'name', 'userId', 'net', 'bonus'])
			->from('money', 'money')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$resultGame = $con->startQuery()
			->select(['content'])
			->from($game)
			->where(['id' => 1])
			->build()
			->query()
			->fetchAssoc()[0]['content'];
		$con->close();
		$resultGames = json_decode(str_replace("\n", "enter", $resultGame), 1);
		if ($resultGames['status'] != 'down') {
			$currentGame = $game.$resultGames['game'];
		}

		foreach ((array) $resultRows as $position => $rows)
		{
			$net += $rows['net'];
			$rows['name'] = json_decode($rows['name'], 1);
			if (!empty($this->search) && stripos($rows['name'].' ', $this->search) === false) {
				unset($resultRows[$position]);
			}
		}

		// if ($status != 'down') {
		// 	$date = date('H') < $resetTime ? date('j', strtotime("-1 days")) : date('j');
		// 	$month = date('H') < $resetTime ? date('n', strtotime("-1 days")) : date('n');
		// 	$year = date('H') < $resetTime ? date('Y', strtotime("-1 days")) : date('Y');
		// }

		$reportRows = $con->startQuery()
			->select(['deposite', 'withdraw', 'bonus'])
			->from('reportMoney', 'money')
			->where(['game' => $currentGame])
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		foreach ((array) $reportRows as $report)
		{
			$resultRows['report']['deposite'] += $report['deposite'];
			$resultRows['report']['withdraw'] += $report['withdraw'];
			$resultRows['report']['bonus'] += $report['bonus'];
		}
		$resultRows['report']['net'] = $net;
		return $resultRows;
	}

	public function setupReport(DatabaseInterface $con, $table)
	{
		$resultRows = $con->startQuery()
			->select(['content'])
			->from($table)
			->where(['id' => 1])
			->build()
			->query()
			->fetchAssoc()[0]['content'];
		$con->close();
		$contents = json_decode(str_replace("\n", "enter", $resultRows), 1);
		$status = $contents['status'];
		$bigLap = $contents['bigLap'];

		$table = 'report'.ucfirst($table);
		if ($status != 'down') {
			$resultRows = $con->startQuery()
				->select(['lap', 'income', 'outcome'])
				->from($table)
				->where(['bigLap' => $bigLap])
				->build()
				->query()
				->fetchAssoc();
			$con->close();
		} else {
			unset($resultRows);
		}

		return $resultRows;
	}

	public function search($search)
	{
		$this->search = $search;
	}

	public function changeMoney(DatabaseInterface $con, $userId, $topup, $bonus, $tags)
	{
		date_default_timezone_set('Asia/Bangkok');
		$game = lcfirst($this->localDatabase->getParameter("game"));
		$data = $con->startQuery()
			->select(['content'])
			->from($game)
			->where(['id' => 1])
			->build()
			->query()
			->fetchAssoc()[0]['content'];
		$con->close();
		$data = str_replace("\n", "enter", $data);
		$datas = json_decode($data, 1);
		if ($datas['status'] == 'down') {
			return -4444;
		}
		$databases = $con->startQuery()
			->select(['net', 'bonus'])
			->from('money', 'money')
			->where(['userId' => $userId])
			->build()
			->query()
			->fetchAssoc()[0];
		$con->close();
		$updateNet = $databases['net']+$topup;
		$deposite = $databases['net'] < 1000 ? $topup : $updateNet;
		if ($topup > 0) {
			$databases['bonus'] += (int) $topup*$bonus/100;
			$bonusText = "+{$bonus}%";
			$bonusRemainingText = '💸'.$databases['bonus'];
		} else {
			$databases['bonus'] = 0;
			$bonusText = '';
			$bonusRemainingText = '';
		}
		$updates = [
			'net'      => $updateNet,
			'bonus'    => $databases['bonus'],
			'deposite' => $deposite,
		];
		$replys = [
			'changeNet' => $topup,
			'bonusText' => $bonusText,
			'remaining' => $updateNet,
			'bonusRemainingText' => $bonusRemainingText,
		];
		sort($tags);
		foreach ((array) $tags as $tag)
		{
			if ($updateNet>=$tag) {
				$updates = array_merge($updates, ['tag' => $tag]);
			}
		}
		if ($datas['recc'] == 'yes') {
			$updates = array_merge($updates, ['count' => 0, 'wage' => 0]);
		}
		$con->startQuery()
			->update($updates)
			->set('money', 'money')
			->where(['userId' => $userId])
			->build()
			->query()
			->close();
		$depoOrWith = $topup < 0 ? 'withdraw' : 'deposite';
		$con->startQuery()
			->insert([
				$depoOrWith => $topup,
				'bonus'     => (int) $bonus,
				'time'      => date('H.i'),
				'date'      => date('j'),
				'month'     => date('n'),
				'year'      => date('Y'),
				'game'      => $game.$datas['game'],
			])
			->into('reportMoney', 'money')
			->build()
			->query()
			->close();
		return $replys;
	}

	public function eachMoney(DatabaseInterface $con, $userId)
	{
		$moneys = $con->startQuery()
			->select(['net', 'bonus'])
			->from('money', 'money')
			->where(['userId' => $userId])
			->build()
			->query()
			->fetchAssoc()[0];
		$con->close();
		return $moneys;
	}

	public function getAllJs($dir)
	{
		$dir = $dir == '/' ? '' : $dir;
		foreach (glob("app/js/*.*") as $filename)
		{
			$temp = explode('/', $filename);
			$filename = end($temp);
			echo "<script src=\"$dir";
			echo "/app/js/".$filename;
			echo "\"></script>\n";
		}
	}

	public function checkLogin(DatabaseInterface $con, $user, $pass)
	{
		
	}
}