<?php
namespace Koob\WheelBundle\Model;

use Koob\InterfaceBundle\WheelInterface;
use Koob\InterfaceBundle\DatabaseInterface;
use Koob\GameBundle\Model\GameModel;

class FunctionModel extends GameModel implements WheelInterface
{
	protected $database = 'wheel';

	protected $attrs = ['reply', 'recc', 'player', 'wage', 'wage+', 'percent', 'rw', 'allDeli', '$'];

	public function rebuildWheel(DatabaseInterface $con)
	{
		$con->startQuery()
			->drop()
			->table($this->database)
			->build()
			->query()
			->close();

		$con->startQuery()
			->create([
				'name'     => 'VARCHAR(255)',
				'userId'   => 'VARCHAR(255) UNIQUE',
				'admin'    => 'VARCHAR(255)',
				'content'  => 'TEXT',
				'result'   => 'TEXT',
				'rollBack' => 'TEXT',
			])
			->table($this->database)
			->build()
			->query()
			->close();

		$datas = [
			'status'  => 'down',
			'game'    => 0,
			'bigLap'  => 1,
			'lap'     => 1,
			'reply'   => 'yes',
			'player'  => 10,
			'wage'    => 100,
			'percent' => 5,
			'rw'  => [100],
		];
		$data = json_encode($datas);
		$con->startQuery()
			->insert([
				'name' => '"index"',
				'userId' => 'index',
				'content' => $data,
			])
			->into($this->database)
			->build()
			->query()
			->close();
	}

	public function wage(DatabaseInterface $con, $text)
	{
		if ($this->datas['status'] == 'down') {
			return;
		} else if ($this->datas['status'] == 'close') {
			return 'ยังไม่เปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			return 'แทงไม่ทัน';
		}
		if ($this->datas['wage+'] == 'yes') {
			$wages = explode($this->datas['allDeli'], $text);
		} else {
			$wages = [$text];
		}
		foreach ((array) $wages as $wage)
		{
			if ($wage < 0 || $wage > $this->datas['player']) {
				$errors[] = "การแทง $wage ผิด แทงได้ไม่เกิน ".$this->datas['player'];
				continue;
			}
			if (in_array($wage, (array) array_keys($this->contents))) {
				continue;
			}
			if (in_array($wage, (array) $this->datas['wageLists'])) {
				$errors[] = "การแทง $wage ซ้ำ";
				continue;
			}
			if (!is_numeric($wage) || $wage != (int) $wage) {
				$errors[] = "การแทง $wage ไม่ใช่ตัวเลข";
				continue;
			}
			if (!empty($this->contents) && $this->datas['wage+'] != 'yes') {
				foreach ((array) $this->contents as $type => $temp)
				{
					unset($this->datas['wageLists'][array_search($type, (array) $this->datas['wageLists'])]);
				}
				unset($this->contents);
			}
			$this->datas['wageLists'][count((array) $this->datas['wageLists'])] = $wage;
			$this->contents[$wage] = 1;
		}
		foreach ((array) $this->contents as $wage)
		{
			$allWageNet += $this->datas['wage'];
		}
		if ($allWageNet <= $this->net+$this->bonus) {
			$errorText = implode(" ", (array) $errors);
		} else {
			$errorText = "เครดิตไม่เพียงพอ❗";
		}
		$clearErrorText = str_replace([' ', "\r\n"], '', $errorText);
		if (empty($clearErrorText)) {
			$content = $this->codename($this->contents);
			$con->startQuery()
				->update(['content' => $content])
				->set($this->database)
				->where(['userId' => $this->userId])
				->build()
				->query()
				->close();
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		$replyTexts = $this->check();
		if ($this->datas['reply'] == 'yes') {
			if (!empty($errorText)) {
				$replyTexts .= "\r\n\r\n$errorText";
			}
		} else if ($this->datas['reply'] == 'flex') {
			if (!empty($clearErrorText)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							$errorText,
							'separator #000000',
						],
					],
				], 'xs');
			}
		} else {
			unset($replyTexts);
		}
		return $replyTexts;
	}

	public function unWage(DatabaseInterface $con, $text)
	{
		if ($this->datas['status'] != 'open') {
			return $this->datas['status'] == 'down' ? '' : "$this->name ปิดรอบแล้ว❗";
		}
		foreach ((array) $this->contents as $type => $wage)
		{
			unset($this->datas['wageLists'][array_search($type, (array) $this->datas['wageLists'])]);
			unset($this->contents[$type]);
			$cancel .= " $type";
		}
		$content = json_encode($this->contents);
		$con->startQuery()
			->update(['content' => $content])
			->set($this->database)
			->where(['userId' => $this->userId])
			->build()
			->query()
			->close();
		$data = json_encode($this->datas);
		$con->startQuery()
			->update(['content' => $data])
			->set($this->database)
			->where(['id' => 1])
			->build()
			->query()
			->close();
		if (!empty($cancel)) {
			if ($this->datas['reply'] == 'flex') {
				$replyTexts = $this->flexMessage([
					'old' => $this->check(),
					'body' => [
						[
							"ยกเลิกขา$cancel",
							"separator #000000",
						],
					],
				], 'xs');
			} else if ($this->datas['reply'] == 'yes') {
				$replyTexts = $this->check()."\r\n\r\nยกเลิกขา$cancel";
			}
		}
		return $replyTexts;
	}

	public function unWageAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		unset($this->datas['wageLists']);
		$con->startQuery()
			->update(['content' => '{}'])
			->set($this->database)
			->where('id>1')
			->build()
			->query()
			->close();
		$data = json_encode($this->datas);
		$con->startQuery()
			->update(['content' => $data])
			->set($this->database)
			->where(['id' => 1])
			->build()
			->query()
			->close();
		return "ยกเลิกการแทงทั้งหมด";
	}

	public function check()
	{
		$replyTexts = $this->flexMessage([
			'altText' => "สรุปการแทงของ $this->name",
			'footer' =>	"{$this->net}💸{$this->bonus}",
			'body' => [
				[
					$this->name,
				],
			],
		], 'xs');
		$isEmpty = 1;
		$text = "$this->name แทง";
		foreach ((array) $this->contents as $type => $wageNet)
		{
			$isEmpty = 0;
			$text .= " $type=$wageNet";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						$type,
						$wageNet,
					],
				],
			], 'xs');
		}
		if ($isEmpty) {
			$text .= "ไม่ได้แทง";
			$replyTexts = $this->flexMessage([
				'old' => $replyTexts,
				'body' => [
					[
						'ไม่ได้แทง',
					],
				],
			], 'xs');
		}
		$text .= "\r\n{$this->net}💸{$this->bonus}";
		$replyTexts = $this->flexMessage([
			'old' => $replyTexts,
			'body' => [
				[
					'separator #000000',
				],
			],
		]);
		if ($this->datas['reply'] == 'yes') {
			$replyTexts = $text;
		}
		return $replyTexts;
	}

	public function checkAll(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}

		$resultRows = $con->startQuery()
			->select(['x.name, x.content, m.userId', 'm.net', 'm.bonus'])
			->from($this->database.' x, money.money m')
			->where('x.userId = m.userId')
			->build()
			->query()
			->fetchAssoc();
		$con->close();
		$replyTexts = $this->flexMessage([
			'altText' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
			'header' => 'สรุปการแทงทั้งหมดรอบที่ '.$this->datas['lap'],
		]);
		foreach ((array) $resultRows as $rows)
		{
			$contents = json_decode($rows['content'], 1);
			$name = json_decode($rows['name'], 1);
			foreach ((array) $contents as $type => $wageNet)
			{
				$reply .= " $type=$wageNet";
			}
			if ($rows['bonus'] > 0) {
				$bonusText = "💸".$rows['bonus']."=".($rows['bonus']+$rows['net']);
			}
			if (!empty($reply)) {
				$replyTexts = $this->flexMessage([
					'old' => $replyTexts,
					'body' => [
						[
							"$name แทง$reply 💰".$rows['net']."{$bonusText}{$suffix}",
						],
					],
				]);
			}
			unset($reply, $bonusText);
		}
		$noPlayerReplys = [
			"type" => "text",
			"text" => "ยังไม่มีผู้เล่นแทง",
		];
		if (empty($replyTexts['messages'][0]['contents']['contents'][0]['body'])) {
			$replyTexts = [
				'messages' => [$noPlayerReplys],
				'raw' => true
			];
		}
		for ($i=1; $i<=$this->datas['player']; $i++)
		{
			if (!in_array($i, $this->datas['wageLists'])) {
				$remain .= " $i";
			}
		}
		$remain = empty($remain) ? "ไม่มีเลขที่ว่าง" : "เลขที่ยังว่างอยู่ $remain";
		$remains = [
			"type" => "text",
			"text" => $remain,
		];
		$replyTexts['messages'][] = $remains;
		return $replyTexts;
	}

	public function openGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'down') {
			$sendText = $this->resultOpen($con, $this->database, $this->datas['game'], $resetTime);
			$game = $this->datas['game'] + 1;
			$replyText = "เริ่มเกมที่ #".$game;
			$this->datas['status'] = 'close';
			$this->datas['game'] = $game;
			$this->datas['bigLap'] = 1;
			$this->datas['lap'] = 1;
			$this->datas['startTime'] = date('H.i');
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else {
			$replyText = "เปิดบ้านแล้ว";
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function openRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			// $replyText[] = $locals['openMessage'];
			// $replyText[] = "เปิดรอบย่อยที่ #".$this->datas['lap'];
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			$this->datas['status'] = 'open';
			unset($this->datas['wageLists']);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'เปิดรอบแล้ว';
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'กรุณาสรุปผลให้เรียบร้อยก่อน';
		}
		return $replyText;
	}

	public function closeRound(DatabaseInterface $con, $locals)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'open') {
			// $replyText[] = $locals['closeMessage'];
			// $replyText[] = 'ปิดรอบย่อยที่ #' . $this->datas['lap'];
			$checkAll = $this->checkAll($con);
			$replyText["raw"] = "true";
			$replyText["messages"] = [
				$checkAll["messages"][0],
				[
					"type" => "image",
					"originalContentUrl" => $locals['pic'],
					"previewImageUrl" => $locals['pic'],
				],
				[
					"type" => "text",
					"text" => "รอบที่ #".$this->datas['lap'],
				],
			];
			$this->datas['status'] = 'check';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'rollBack' => $data])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'down') {
			$replyText = 'โปรดเช็คสถานะ';
		} else {
			$replyText = 'ปิดรอบแล้ว';
		}
		return $replyText;
	}

	public function restRound(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultRest($con, $this->database, $this->datas['game'], $this->datas['bigLap'], $resetTime, $this->datas['startTime']);
			$bigLap = $this->datas['bigLap'];
			$replyText = 'ปิดรอบใหญ่ที่ #'.$bigLap;
			$this->datas['lap'] = 1;
			$this->datas['bigLap'] = $bigLap + 1;
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'admin' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['count' => 0, 'wage' => 0, 'deposite' => 0])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else {
			$replyText = 'โปรดเช็คสถานะ';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function closeGame(DatabaseInterface $con, $resetTime)
	{
		if (!in_array($this->admin, ['yes', 'super'])) {
			return;
		}
		if ($this->datas['status'] == 'close') {
			$sendText = $this->resultClose($con, $this->database, $this->datas['game'], $resetTime);
			$replyText = 'ปิดบ้านแล้ว';
			$this->rebuildReport($con, $this->database);
			$this->datas['status'] = 'down';
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'admin' => null, 'rollBack' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
			$whereText = $con->select(['userId'])
				->from($this->database)
				->where('id > 1')
				->build()
				->getSql();
			$con->startQuery()
				->update(['tag' => 0, 'count' => 0, 'wage' => 0, 'deposite' => 0, 'rollBack' => null])
				->set('money', 'money')
				->where("userId in ($whereText)")
				->build()
				->query()
				->close();
		} else if ($this->datas['status'] == 'open') {
			$replyText = 'ยังไม่ปิดรอบ';
		} else if ($this->datas['status'] == 'check') {
			$replyText = 'ยังไม่สรุป';
		} else {
			$replyText = 'ปิดบ้านแล้ว';
		}
		$replyTexts = [$replyText, $sendText];
		return $replyTexts;
	}

	public function saveResult(DatabaseInterface $con, $text)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] == 'down' || $this->datas['status'] == 'close') {
			return;
		}
		if ($this->datas['status'] != 'check') {
			return 'กรุณาปิดรอบก่อน';
		}
		$texts = substr($text, 1);
		foreach (explode(',', $texts) as $text){
			if (!is_numeric($text) || $text > $this->datas['player'] || $text <= 0) {
				$error = 1;
			} else {
				$replyText .= " $text";
				$results[] = $text;
			}
		}
		if (count((array) $results) != count((array) $this->datas['rw'])) {
			$error = 1;
		}
		if ($error == 1) {
			$replyText = "สรุปผิด (".count((array) $this->datas['rw']).")";
		} else {
			$replyText = "สรุป$replyText\r\n\r\nยันยืนผลสรุป yes หรือ @ok";
			$result = json_encode($results);
			$con->startQuery()
				->update(['result' => $result])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function submit(DatabaseInterface $con)
	{
		if (!in_array($this->admin, ['yes', 'super']) || $this->datas['status'] != 'check') {
			return;
		}
		if (empty($this->results)) {
			return 'สรุปผิด';
		} else {
			$con->startQuery()
				->update(['rollBack' => null])
				->set($this->database)
				->build()
				->query()
				->close();
			$recieveDatabases = $con->startQuery()
				->select(['id', 'content'])
				->from($this->database)
				->where('content IS NOT NULL')
				->and('content != ""')
				->and('content != "{}"')
				->build()
				->query()
				->fetchAssoc();
			$con->close();
			foreach ((array) $recieveDatabases as $rows)
			{
				$con->startQuery()
					->update(['rollBack' => $rows['content']])
					->set($this->database)
					->where(['id' => $rows['id']])
					->build()
					->query()
					->close();
			}
			$con->startQuery()
				->update(['rollBack' => null])
				->set('money', 'money')
				->build()
				->query()
				->close();
			$replyText = $this->result($con, $this->database);
			$this->datas['lap']++;
			$this->datas['status'] = 'close';
			unset($this->results);
			$data = json_encode($this->datas);
			$con->startQuery()
				->update(['content' => $data, 'result' => null])
				->set($this->database)
				->where(['id' => 1])
				->build()
				->query()
				->close();
		}
		return $replyText;
	}

	public function setup(DatabaseInterface $con, $admins, $force)
	{
		if (!in_array($this->userId, $admins)) {
			return;
		}
		$database = $con->getDatabase();

		$existedDatabases = $con->startQuery()
			->select('SCHEMA_NAME')
			->from('SCHEMATA', 'information_schema')
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array($database, $existedDatabases)) {
			$con->startQuery()
				->create()
				->database($database)
				->build()
				->query()
				->close();
		}

		if (!in_array('money', $existedDatabases)) {
			$con->startQuery()
				->create()
				->database('money')
				->build()
				->query()
				->close();
		}

		$allTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => $database])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		$moneyTables = $con->startQuery()
			->select('TABLE_NAME')
			->from('TABLES', 'information_schema')
			->where(['TABLE_SCHEMA' => 'money'])
			->build()
			->query()
			->fetchAssoc(true);
		$con->close();

		if (!in_array('money', $moneyTables)) {
			$this->rebuildMoney($con);
			$install .= ' money';
		}
		if (!in_array('reportMoney', $moneyTables)) {
			$this->rebuildReportMoney($con);
			$install .= ' reportMoney';
		}
		if (!in_array('login', $allTables)) {
			$this->rebuildLogin($con);
			$install .= ' login';
		}
		if ($force === true || !in_array($this->database, $allTables)) {
			$this->rebuildWheel($con);
			$install .= ' wheel';
		}
		if ($force === true || !in_array('reportWheel', $allTables)) {
			$this->rebuildReport($con, $this->database);
			$install .= ' reportWheel';
		}
		if ($force === true || !in_array('reportAllWheel', $allTables)) {
			$this->rebuildAllReport($con, $this->database);
			$install .= ' reportAllWheel';
		}

		if (!empty($install)) {
			$replyText = "ติดตั้ง$install สำเร็จ";
		} else {
			$replyText = "มีฐานข้อมูลอยู่แล้ว";
		}
		return $replyText;
	}

	protected function rule()
	{
		$allWage = $this->datas['wage']*(count($this->datas['wageLists']));
		$percent = $allWage*(1-$this->datas['percent']/100);
		for ($i=1; $i<=$this->datas['player']; $i++)
		{
			$reduces[$i] = $this->datas['wage']*-1;
		}
		$results = $this->results;
		foreach ((array) $results as $n => $result)
		{
			if (stripos($this->datas['rw'][$n], '%')) {
				$this->datas['rw'][$n] = substr($this->datas['rw'][$n], 0, -1);
				$rewards[$result] += $percent*($this->datas['rw'][$n])/100;
			} else {
				$rewards[$result] += $this->datas['rw'][$n];
			}
		}
		foreach (array_unique((array) $results) as $result)
		{
			$rewards[$result] -= $this->datas['wage'];
		}
		return [$rewards, $results, $reduces];
	}
}